@@ -35,7 +35,7 @@ class GenericImporterV0_2_0 {
             
             
             Log::write(Log::LOGLEVEL_INFO, 'Checking for existence of default project...');
-            $defaultProject=project::getByName(project::DEFAULTPROJECT);
+            $defaultProject=project::getByName(baseproject::DEFAULTPROJECT);
             Log::write(Log::LOGLEVEL_INFO, '...Exists.');
             
             $json=json_decode($jsonString, true);
@@ -70,7 +70,7 @@ class GenericImporterV0_2_0 {
             foreach($trace as $t){
                 $traceline='';
                 $line=(isset($t['line'])) ? $t['line'] : '--';
-                $file=(isset($t['file'])) ? str_replace($_SERVER['DOCUMENT_ROOT'], '', (str_replace('\\','/',$t['file']))) : '--';
+                $file=(isset($t['file'])) ? str_replace(config::getWwwRoot(), '', (str_replace('\\','/',$t['file']))) : '--';
                 if (array_key_exists('class',$t)){
                     $traceline=sprintf("%s:%s %s::%s",
                         $file,
@@ -110,7 +110,7 @@ class GenericImporterV0_2_0 {
         }
     }
     
-    private static function importPlate($plate){
+    private static function importPlate($plate): void {
         Log::write(Log::LOGLEVEL_DEBUG, 'In importPlate');
         static::$currentPlate=null;
         static::$currentPlateOwner=null;
@@ -148,7 +148,7 @@ class GenericImporterV0_2_0 {
 
         $owner=static::importUser($owner);
         static::$currentPlateOwner=$owner;
-        $defaultProject=project::getByName(project::DEFAULTPROJECT);
+        $defaultProject=project::getByName(baseproject::DEFAULTPROJECT);
         $projectId=$defaultProject['id'];
 
         $limsPlate=plate::getByName($barcode);
@@ -194,8 +194,7 @@ class GenericImporterV0_2_0 {
         }
         
         Log::write(Log::LOGLEVEL_DEBUG, 'Returning from importPlate');
-        return $limsPlate;
-    }
+	}
     
     private static function importPlateType($plateType){
         Log::write(Log::LOGLEVEL_DEBUG, 'In importPlateType');
@@ -514,11 +513,7 @@ class GenericImporterV0_2_0 {
         } else {
             $projectOwner=static::$currentPlateOwner;
             $existingProject=null;
-            try {
-                $existingProject=project::getByName($proteinName);
-            } catch(NotFoundException $e){
-                //This is what we want.
-            }
+			$existingProject=project::getByName($proteinName);
             if($existingProject){
                 throw new Exception($proteinName.': Protein does not exist but there is a project with that name.');
             }
@@ -608,7 +603,7 @@ class GenericImporterV0_2_0 {
         return $construct;
     }
 
-    private static function getDrives(){
+    private static function getDrives(): void {
         if(!static::$drives){
             static::$drives=array();
             try {
@@ -622,7 +617,6 @@ class GenericImporterV0_2_0 {
                 Log::write(Log::LOGLEVEL_WARN, $e->getMessage());
             }
         }
-        return static::$drives;
-    }
+	}
     
 }
