@@ -1,6 +1,8 @@
-<?php class audiorecordingfile extends audiorecording {
+<?php use JetBrains\PhpStorm\NoReturn;
+
+class audiorecordingfile extends audiorecording {
 	
-	public static function getById($id){
+	#[NoReturn] public static function getById($id): ?array {
 		$recording=audiorecording::getById($id);
 		$path=rtrim(config::get('core_filestore'),'/').'/files/project'.$recording['projectid'].'/audio/recording'.$id;
 		$file=fopen($path,'rb');
@@ -10,13 +12,13 @@
 		exit;
 	}
 	
-	public static function create($request=array()){
+	public static function create(array $request=array()): array {
 		throw new BadRequestException('Create an audiorecording instead');
 	}
-	public static function update($id,$request=array()){
+	public static function update($id,$request=array()): array {
 		throw new BadRequestException('Update the audiorecording instead');
 	}
-	public static function delete($id){
+	public static function delete($id): array {
 		throw new BadRequestException('Delete the audiorecording instead');
 	}
 	
