@@ -2,20 +2,24 @@
 
 	protected static $fields=array();
 	protected static $helpTexts=array();
-	
-	public static function getFieldValidations(){
+	protected static $aliasedColumns=array(); //Returned in $adminSelect/$normalSelect but NOT in called class's columns
+
+	public static function getFieldValidations(): array {
 		return static::$fields; //static, not self - use the child class's.
 	}
-	public static function getFieldHelpTexts(){
-		return static::$helpTexts; //static, not self - use the child class's.
-	}
-	
-	public static $defaultSortOrder='name ASC';
+    public static function getFieldHelpTexts(): array {
+        return static::$helpTexts; //static, not self - use the child class's.
+    }
+    public static function getAliasedColumns(){
+        return static::$aliasedColumns; //static, not self - use the child class's.
+    }
+
+    public static $defaultSortOrder='name ASC';
 
 	/**
 	 * The terms used where this class is included in a "get by name like" search.
 	 * By default, this is the name property. However, in some cases it may be useful
-	 * to override this so that search is done one a more relevant property.
+	 * to override this so that search is done on a more relevant property.
 	 * @var string
 	 */
 	protected static $searchClause='[[classname]].name';
@@ -31,10 +35,9 @@
      * @param bool $forceSharedProject
      * @return string
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      */
-	protected static function getProjectClause($accessType, $forceSharedProject=false){
+	protected static function getProjectClause(string $accessType, bool $forceSharedProject=false): string {
 		return database::getProjectClause($accessType, $forceSharedProject);
 	}
 
@@ -82,60 +85,121 @@
 		return database::getLastInsertId();
 	}
 
-    /**
-     * Returns the record with the specified ID.
-     * @param int $id The ID of the desired record
-     * @return array The record
-     * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
-     */
-	public static function getById($id){
+	/**
+	 * @throws ServerException
+	 * @throws BadRequestException
+	 * @throws NotFoundException
+	 */
+	public static function getUuid($id, $setIfNull=false) {
+		$className=get_called_class();
+		$obj=static::getById($id);
+		if(!$obj){
+			throw new NotFoundException("No $className with ID $id");
+		}
+		if(empty($obj['uuid'])){
+			if(!$setIfNull){ return null; }
+			$obj['uuid']=static::setUuid($id);
+		}
+		return $obj['uuid'];
+	}
+
+	/**
+	 * @throws ServerException
+	 * @throws NotFoundException
+	 * @throws BadRequestException
+	 */
+	public static function setUuid($id, $uuid=null) {
+		$className=get_called_class();
+		$obj=static::getById($id);
+		if(!$obj){
+			throw new NotFoundException("No $className with ID $id");
+		}
+		if(!empty($obj['uuid'])){ return $obj['uuid']; }
+		if(!$uuid){
+			$uuid=baseobject::generateUuid();
+		}
+		try {
+			database::query(
+				"UPDATE $className SET uuid=:uuid WHERE id=:id",
+				[':uuid'=>$uuid, ':id'=>$id]
+			);
+		} catch (ServerException $e){
+			throw new ServerException("No uuid column in $className? ".$e->getMessage());
+		}
+		return $uuid;
+	}
+
+	public static function generateUuid() {
+		$timestamp = intval(microtime(true) * 1000);
+
+		return sprintf('%02x%02x%02x%02x-%02x%02x-%04x-%04x-%012x',
+			// first 48 bits are timestamp based
+			($timestamp >> 40) & 0xFF,
+			($timestamp >> 32) & 0xFF,
+			($timestamp >> 24) & 0xFF,
+			($timestamp >> 16) & 0xFF,
+			($timestamp >> 8) & 0xFF,
+			$timestamp & 0xFF,
+
+			// 16 bits: 4 bits for version (7) and 12 bits for rand_a
+			mt_rand(0, 0x0FFF) | 0x7000,
+
+			// 16 bits: 4 bits for variant where 2 bits are fixed 10 and next 2 are random to get (8-9, a-b)
+			// next 12 are random
+			mt_rand(0, 0x3FFF) | 0x8000,
+			// random 48 bits
+			mt_rand(0, 0xFFFFFFFFFFFF),
+		);
+	}
+
+	public static function getByUuid($uuid): ?array {
+		return static::getBy('uuid',$uuid);
+	}
+
+	/**
+	 * Returns the record with the specified ID.
+	 * @param int $id The ID of the desired record
+	 * @return array|null The record
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function getById($id): ?array {
+		return static::getBy('id', $id);
+	}
+
+	/**
+	 * Returns the record with the specified name.
+	 * @param string $name The name of the desired record
+	 * @return array|null The record
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function getByName($name): ?array {
+		return static::getBy('name',$name);
+	}
+
+	private static function getBy($key, $value): ?array {
 		$className=get_called_class();
 		if(session::isAdmin()){
 			$sqlStatement=static::$adminSelect;
 		} else {
 			$sqlStatement=static::$normalSelect;
 		}
-		$sqlStatement.=' AND [[classname]].id=:id';
+		$sqlStatement.=" AND [[classname]].$key=:$key";
 		$sqlStatement=str_replace('[[classname]]', $className, $sqlStatement);
-		$sqlStatement.=static::getProjectClause('read');
-		$params=array(':id'=>$id);
+		$sqlStatement.=static::getProjectClause('readOne');
+		$params=array(":$key"=>$value);
 		return database::queryGetOne($sqlStatement,$params);
 	}
 
-
-    /**
-     * Returns the record with the specified name.
-     * @param string $name The name of the desired record
-     * @return array The record
-     * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
-     */
-	public static function getByName($name){
-		$className=get_called_class();
-		if(session::isAdmin()){
-			$sqlStatement=static::$adminSelect;
-		} else {
-			$sqlStatement=static::$normalSelect;
-		}
-		$sqlStatement.=' AND [[classname]].name=:name';
-		$sqlStatement=str_replace('[[classname]]', $className, $sqlStatement);
-		$sqlStatement.=static::getProjectClause('read');
-		$params=array(':name'=>$name);
-		return database::queryGetOne($sqlStatement,$params);
-	}
-
-    /**
-     * @param string $searchTerms
-     * @param array $request
-     * @return array
-     * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
-     */
-	public static function getByNameLike($searchTerms,$request=array()){
+	/**
+	 * @param string $searchTerms
+	 * @param array $request
+	 * @return array|null
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function getByNameLike(string $searchTerms, array $request=array()): ?array {
 		$className=get_called_class();
 		if(session::isAdmin()){
 			$sqlStatement=static::$adminSelect;
@@ -187,17 +251,23 @@
 			if(!class_exists($className)){
 				throw new BadRequestException('Class '.$className.' does not exist');
 			}
+			if('project'===$className){
+				$sqlStatement='SELECT "project" AS objecttype, id, name, id AS projectid, name AS projectname, isarchived FROM project WHERE LOWER(name) LIKE LOWER(:terms'.$counter.')';
+				if(!session::isAdmin()){
+					$projectIds=session::getReadProjects();
+					$sqlStatement.=' AND id IN('.implode(',',$projectIds).')';
+				}
+				$parts[]=$sqlStatement;
+				$params[':terms'.$counter]=$innerSearchTerms;
+				continue;
+			}
 			if(session::isAdmin()){
 				$sqlStatement=static::$adminSelect;
 			} else {
 				$sqlStatement=static::$normalSelect;
 			}
 
- 			if(isset($className::$searchClause)){
- 			    $searchClause=$className::$searchClause;
- 			} else {
- 			    $searchClause=baseobject::$searchClause;
- 			}
+			$searchClause = $className::$searchClause ?? baseobject::$searchClause;
 			
 			$sqlStatement=str_replace('SQL_CALC_FOUND_ROWS', '', $sqlStatement);
 			$sqlStatement=str_replace('SELECT ', 'SELECT "[[classname]]" AS objecttype, ', $sqlStatement);
@@ -231,12 +301,11 @@
 
     /**
      * Generates the SQL condition for whether objects can be seen in a multi-class search. The default implementation
-     * returns database::getProjectClause("read"), something line ' AND project.id IN(1,3,4) '. Any overriding implementation
+     * returns database::getProjectClause("read"), something like ' AND project.id IN(1,3,4) '. Any overriding implementation
      * must produce similar SQL for the search query to be valid.
      * @return string
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      */
 	public static function getReadConditionsForMultiClassSearch(){
 		return static::getProjectClause('read');
@@ -249,15 +318,15 @@
      * @param array $request The request object, used for filtering, limit clause, etc.
      * @return array
      * @throws BadRequestException
-     * @throws NotFoundException if the record does not exist or the user cannot read it
-     * @throws ServerException
+	 * @throws ServerException
      */
-	public static function getByProperty($key,$value,$request=array()){
+	public static function getByProperty($key,$value,$request=array()): ?array {
 		$className=get_called_class();
 		//verify key in fields array
 		$keys=array_keys(self::getFieldValidations());
-	
-		if(!in_array($key, $keys) &&'projectid'!=$key){
+        $aliasedColumns=static::getAliasedColumns();
+
+        if(!in_array($key, $keys) &&'projectid'!=$key && !in_array($key, $aliasedColumns)){
 			throw new BadRequestException('Property '.$key.' not recognised on '.$className);
 		}
 		if(session::isAdmin()){
@@ -265,27 +334,29 @@
 		} else {
 			$sqlStatement=static::$normalSelect;
 		}
-		$sqlStatement.=' AND [[classname]].'.$key.'<=>:val ';
+        if(in_array($key,$aliasedColumns)){
+            $sqlStatement.=" AND $key<=>:$key ";
+        } else {
+            $sqlStatement.=" AND [[classname]].$key<=>:$key ";
+        }
 		$sqlStatement=str_replace('[[classname]]', $className, $sqlStatement);
-		//		$sqlStatement='SELECT x.*,project.name AS projectname, project.isarchived AS isarchived FROM '.$className.' AS x, project WHERE project.id=x.projectid AND x.'.$key.'=:val';
 		$sqlStatement.=static::getProjectClause('read');
         $sqlStatement.=database::getFilterClause($request, $className);
         $sqlStatement.=database::getOrderClause($request, $className);
 		$sqlStatement.=database::getLimitClause($request);
-		$params=array(':val'=>$value);
+		$params=array(":$key"=>$value);
 		return database::queryGetAll($sqlStatement,$params);
 	}
 
-    /**
-     * Returns the records with the specified property.
-     * @param array $keyValuePairs
-     * @param array $request The request object, used for filtering, limit clause, etc.
-     * @return array
-     * @throws BadRequestException
-     * @throws ServerException
-     * @throws NotFoundException
-     */
-	public static function getByProperties($keyValuePairs,$request=array()){
+	/**
+	 * Returns the records with the specified property.
+	 * @param array $keyValuePairs
+	 * @param array $request The request object, used for filtering, limit clause, etc.
+	 * @return array|null
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function getByProperties(array $keyValuePairs, array $request=array()): ?array {
 		$className=get_called_class();
 		if(session::isAdmin()){
 			$sqlStatement=static::$adminSelect;
@@ -293,14 +364,19 @@
 			$sqlStatement=static::$normalSelect;
 		}
 		$keys=array_keys(self::getFieldValidations());
+        $aliasedColumns=self::getAliasedColumns();
 		$params=array();
 		foreach($keyValuePairs as $key=>$value){
-			if(!in_array($key, $keys) &&'projectid'!=$key){
+            if(!in_array($key, $keys) &&'projectid'!=$key && !in_array($key, $aliasedColumns)){
 				throw new BadRequestException('Property '.$key.' not recognised on '.$className);
 			}
-			$sqlStatement.=' AND [[classname]].'.$key.'<=>:'.$key.' ';
+            if(in_array($key,$aliasedColumns)){
+                $sqlStatement.=" AND $key<=>:$key ";
+            } else {
+                $sqlStatement.=" AND [[classname]].$key<=>:$key ";
+            }
 			$sqlStatement=str_replace('[[classname]]', $className, $sqlStatement);
-			$params[':'.$key]=$value;
+			$params[":$key"]=$value;
 		}
 		$sqlStatement.=static::getProjectClause('read');
         $sqlStatement.=database::getFilterClause($request, $className);
@@ -315,9 +391,8 @@
      * @return array
      * @throws BadRequestException
      * @throws ServerException
-     * @throws NotFoundException
-     */
-	public static function getAll($request=array()){
+	 */
+	public static function getAll(array $request=array()): ?array {
 		$className=get_called_class();
 		//verify key in fields array
 		if(session::isAdmin()){
@@ -341,8 +416,7 @@
      * @param array $request
      * @return array
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      */
 	public static function getFiles($id, $request=array()){
         return file::getByProperty('parentid',$id,$request);
@@ -358,8 +432,7 @@
      * @throws ServerException
      */
     public static function getNotes($id, $request=array()){
-        $request['pagenumber']='1';
-        $request['pagesize']='100000';
+        $request['all']='1';
         $request['sortby']='entered';
         return note::getByProperty('parentid',$id,$request);
     }
@@ -374,13 +447,25 @@
      * @throws ServerException
      */
     public static function getRecordings($id, $request=array()){
-        $request['pagenumber']='1';
-        $request['pagesize']='100000';
+        $request['all']='1';
         $request['sortby']='id';
         $request['sortdescending']='yes';
         return audiorecording::getByProperty('parentid',$id,$request);
     }
 
+    /**
+     * Forces the files, notes, and audio recordings attached to an object to have the same project ID
+     * as that object.
+     * @param $parentParentId
+     * @throws BadRequestException
+     * @throws NotFoundException
+     * @throws ServerException
+     */
+    public static function updateFileNoteAndRecordingProjectIds($parentParentId){
+        file::setProjectIdsToMatchParent($parentParentId);
+        note::setProjectIdsToMatchParent($parentParentId);
+        audiorecording::setProjectIdsToMatchParent($parentParentId);
+    }
 
     /**
      * Creates a new record
@@ -391,7 +476,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function create($request=array()){
+	public static function create(array $request=array()): array {
 	    $className=get_called_class();
 	    return baseobject::doCreate($request, $className);
 	}
@@ -406,7 +491,7 @@
      * @throws ServerException
      */
 	public static function createByClassName($request,$className){
-	    return baseobject::doCreate($request, $className);
+	    return static::doCreate($request, $className);
 	}
 
     /**
@@ -424,6 +509,13 @@
 		if(!isset($request['projectid'])){
 			throw new BadRequestException('Project ID not provided.');
 		}
+		$canCreateInProject=project::canCreateInProject($request['projectid']);
+		if(!$canCreateInProject){
+			if(!project::canReadProject($request['projectid'])){
+				throw new NotFoundException('Project not found');
+			}
+			throw new ForbiddenException('You cannot create in that project');
+		}
 		$fieldValidations=forward_static_call($className.'::getFieldValidations');
 		foreach($fieldValidations as $fname=>$validations){
 			if(isset($request[$fname])){
@@ -435,6 +527,7 @@
 		}
 		$keys='`'.implode('`,`', $fields).'`';
 		$values=':'.implode(',:',$fields);
+		static::beforeCreateHook($request, $className);
 		$baseObjectId=self::createBaseObject($request['projectid'], $className);
         /** @noinspection SqlInsertValues */
         /** @noinspection SqlResolve */
@@ -447,15 +540,20 @@
 		}
 		database::query($sqlStatement,$params);
 		if('file'==$className){
-		    return array('created'=>array('id'=>$baseObjectId, 'projectid'=>$request['projectid']) );
-		}
-		return array('type'=>$className, 'created'=>forward_static_call(array($className,'getById'), $baseObjectId));
+            $created=array('id'=>$baseObjectId, 'projectid'=>$request['projectid']);
+		    $ret=array('type'=>$className, 'created'=>$created);
+		} else {
+		    $created=forward_static_call(array($className,'getById'), $baseObjectId);
+            $ret=array('type'=>$className, 'created'=>$created);
+        }
+        static::afterCreateHook($created, $className);
+		return $ret;
 	}
 
 
     /**
      * Updates one or more fields for a given record. Fields are updated if the field name is found in the fields array of the calling class.
-     * @param int id The ID of the record
+     * @param int $id The ID of the record
      * @param array $request
      * @return array The updated record
      * @throws BadRequestException if one or more fields is invalid
@@ -463,12 +561,13 @@
      * @throws ServerException
      * @throws NotFoundException
      */
-	public static function update($id,$request=array()){
+	public static function update($id,$request=array()): array {
 		$className=get_called_class();
 		if(isset($request['_classname'])){
 		    $className=$request['_classname'];
 		    unset($request['_classname']);
 		}
+
 		if(!preg_match('/^[A-Za-z0-9_-]*$/',$className)){
 			throw new BadRequestException('Bad class name got to baseobject::update');
 		}
@@ -487,11 +586,16 @@
 				throw new ForbiddenException("You do not have permission to update that record");	
 			}
 		}
-		//now we can get on with it
-		foreach(static::getFieldValidations() as $field=>$validations){
+
+        //now we can get on with it
+        $fieldValidations=forward_static_call_array(array($className, 'getFieldValidations'), []);
+		foreach($fieldValidations as $field=>$validations){
 			if(!preg_match('/^[A-Za-z0-9_-]*$/',$field)){
 				throw new BadRequestException('Bad field name got to baseobject::update');
 			}
+		}
+		static::beforeUpdateHook($id, $request, $className);
+        foreach($fieldValidations as $field=>$validations){
 			if(isset($request[$field])){
 				validator::validate($field,$request[$field],$validations); //will throw exception if invalid
 				$params=array(
@@ -510,6 +614,7 @@
 			':lastedittime'=>$now->format('Y-m-d H:i:s')
 		);
 		database::query('UPDATE baseobject SET lasteditor=:lasteditor, lastedittime=:lastedittime WHERE id=:id', $params);
+		static::afterUpdateHook($id, $className);
 		return array('updated'=>static::getById($id));
 	}
 
@@ -517,18 +622,18 @@
     /**
      * Deletes the record with the specified ID, or throws an exception. Note that this does an SQL DELETE of the row in the
      * baseobject table ONLY - it relies on the correct ON DELETE CASCADE behaviour being set in the child table definitions.
-     * @param int id The database ID of the object to delete
+     * @param int $id The database ID of the object to delete
      * @return array
      * @throws BadRequestException
      * @throws ForbiddenException if the user can read but not delete the record
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function delete($id){
+	public static function delete($id): array {
 	    $className=get_called_class();
         /** @noinspection PhpUndefinedMethodInspection */
         if(!session::isAdmin() && !$className::canDelete($id)){
-	        $forceSharedProject=(isset($className::$sharedProjectOverride) && $className::$sharedProjectOverride);
+			$forceSharedProject=(isset($className::$sharedProjectOverride) && $className::$sharedProjectOverride);
             /** @noinspection SqlResolve */
             $sqlStatement='SELECT projectid FROM '.$className.',project
 					WHERE '.$className.'.projectid=project.id
@@ -542,20 +647,24 @@
 	            throw new ForbiddenException("You do not have permission to delete that record");
 	        }
 	    }
+        $object=static::getById($id);
+        static::beforeDeleteHook($object, $className);
 		$sqlStatement='DELETE FROM baseobject WHERE id=:id';
 		$params=array(':id'=>$id);
 		database::query($sqlStatement,$params);
 		//if affected rows, OK, else
 		//get by id
+        static::afterDeleteHook($object, $className);
 		return array('deleted'=>$id);
 	}
 
-    /**
-     * Whether the current user can create records in any project.
-     * This default implementation returns true if the user (a) is an administrator, or (b) has create permissions on any project.
-     * @throws BadRequestException
-     */
-	public static function canCreate(){
+	/**
+	 * Whether the current user can create records in any project.
+	 * This default implementation returns true if the user (a) is an administrator, or (b) has "create" permissions on any project.
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function canCreate(): bool {
 		return session::isAdmin() || !empty(session::getCreateProjects());
 	}
 
@@ -566,10 +675,8 @@
      * @return bool
      * @throws BadRequestException
      * @throws ServerException
-     * @throws NotFoundException
-     */
-	public static function canUpdate($id){
-	    if(session::isAdmin()){ return true; }
+	 */
+	public static function canUpdate($id): bool {
 		$projects=session::getUpdateProjects();
 		$item=static::getById($id);
 		return in_array($item['projectid'], $projects);
@@ -578,18 +685,18 @@
 
     /**
      * Whether the current user can delete the record with this ID.
-     * This default implementation returns true if the user (a) is an administrator, or (b) has delete permissions on the record's project.
+     * This default implementation returns true if the user (a) is an administrator, or (b) has "delete" permissions on the record's project.
      * @param $id
      * @return bool
      * @throws BadRequestException
      * @throws ServerException
-     * @throws NotFoundException
-     */
-	public static function canDelete($id){
+	 */
+	public static function canDelete($id): bool {
 		if(session::isAdmin()){ return true; }
-		if(!config::get('allow_delete')){ return false; }
+		if(!config::get('delete_allowed')){ return false; }
 		$projects=session::getDeleteProjects();
 		$item=static::getById($id);
+		if(!$item){ return false; }
 		return in_array($item['projectid'], $projects);
 	}
 
@@ -635,7 +742,6 @@
 	    if(!static::canMakeQrCodes()){ return false; }
 	    $saveTo=rtrim(config::get('core_filestore'),'/').'/qr_'.str_replace('.','_',$filename).'_'.str_replace(' ','_',microtime()).'.png';
 
-	    include_once(__DIR__.'/../../vendor/kreativekorp/barcode/barcode.php');
 	    $generator=new barcode_generator();
 	    $image=$generator->render_image('qr-m', $url, array());
 	    imagepng($image, $saveTo);
@@ -653,5 +759,113 @@
 	    session::revertAdmin();
 	    return $file;
 	}
-	
+
+    //Hook functions
+
+    /**
+     * @param $request
+     * @throws ServerException
+     */
+    public static function beforeCreateHook($request, $className=null){
+        static::doHook('before','create',array(
+            'request'=>$request
+        ), $className);
+    }
+
+    /**
+     * @param $createdObject
+     * @throws ServerException
+     */
+    public static function afterCreateHook($createdObject, $className=null){
+        static::doHook('after','create',array(
+            'created'=>$createdObject,
+            'id'=>$createdObject['id']
+        ), $className);
+    }
+
+    /**
+     * @param $id
+     * @param $request
+     * @throws ServerException
+     */
+    public static function beforeUpdateHook($id, $request, $className=null){
+        static::doHook('before','update',array(
+            'id'=>$id,
+            'request'=>$request
+        ), $className);
+    }
+
+    /**
+     * @param $id
+     * @throws BadRequestException
+     * @throws NotFoundException
+     * @throws ServerException
+     */
+    public static function afterUpdateHook($id, $className=null){
+        $updatedObject=static::getById($id);
+        static::doHook('after','update',array(
+            'updated'=>$updatedObject,
+            'id'=>$id
+        ), $className);
+    }
+
+	/**
+	 * @param $objectToDelete
+	 * @param null $className
+	 * @throws ServerException
+	 */
+    public static function beforeDeleteHook($objectToDelete, $className=null){
+		if(!$objectToDelete){ return; }
+        static::doHook('before','delete',array(
+            'toDelete'=>$objectToDelete,
+			//TODO line below causes warning on containercontent delete (I think)
+            'id'=>$objectToDelete['id']
+        ), $className);
+    }
+
+	/**
+	 * @param $deletedObject
+	 * @param null $className
+	 * @throws ServerException
+	 */
+    public static function afterDeleteHook($deletedObject, $className=null){
+		if(!$deletedObject){ return; }
+        static::doHook('after','delete',array(
+            'deleted'=>$deletedObject,
+            'id'=>$deletedObject['id']
+        ), $className);
+    }
+
+    /**
+     * Performs actions before calling the IceBear API method.
+     * @param $when string 'before' or 'after'
+     * @param $action string What is being done (create,update,delete)
+     * @param $args array The arguments to be made available in the hook file
+     * @param null|string $className
+     * @throws ServerException
+     */
+    private static function doHook($when, $action, $args, $className=null){
+        if(null===$className){ $className=get_called_class(); }
+        if(!in_array($when,array('before','after'))){
+            throw new ServerException('Bad when in doHook: '.$when.' (should be one of before,after)');
+        }
+        $hookActions=array('create','update','delete');
+        if(!in_array($action, $hookActions)){
+            throw new ServerException('Bad action in doHook: '.$action.' (should be one of '.implode(',',$hookActions).')');
+        }
+        $filename=$className.'_'.$when.'_'.$action;
+        if(!preg_match('/^[a-z]+_[a-z]+_[a-z]+$/', $filename)){
+            throw new ServerException('Bad hook filename in doHook: '.$filename);
+        }
+        $hookFile=config::getWwwRoot().'/api/_hooks/'.$filename;
+        if(file_exists($hookFile.'.php')){
+			include_once($hookFile.'.php');
+        }
+        $count=1;
+        while(file_exists($hookFile.'_'.$count.'.php')){
+			include_once($hookFile.'_'.$count.'.php');
+            $count++;
+        }
+    }
+
 }
