@@ -4,7 +4,7 @@
      * @param array $request
      * @return null
      */
-    public static function getAll($request=array()){
+    public static function getAll(array $request=array()): ?array {
         return null;
     }
 
