@@ -24,8 +24,8 @@ class updater extends baseobject {
     private static $preservePaths=array(
         'conf/config.ini', //Don't wipe out the IceBear database credentials or other configuration!
         'genericimporter/config.ini', //Don't wipe out the IceBear database credentials or other configuration!
-        'install/', //Don't write an install directory, IceBear is already installed!
-        'installer/', //Don't write an install directory, IceBear is already installed!
+        'install/', //Don't write an installation directory, IceBear is already installed!
+        'installer/', //Don't write an installation directory, IceBear is already installed!
         //'classes/core/updater.class.php', //This file
         //'client/templates/core/config/list.php', //The config page
     );
@@ -38,7 +38,7 @@ class updater extends baseobject {
      * @throws ForbiddenException
      * @throws ServerException
      */
-    public static function getAll($request=array()){
+    public static function getAll($request=array()): ?array {
         if(!session::isAdmin()){ throw new ForbiddenException('Only administrators can install updates'); }
         if(isset($request['getupdatelog'])){
             //Retrieve the log of the ongoing update
@@ -66,7 +66,7 @@ class updater extends baseobject {
      * @throws ForbiddenException
      * @throws ServerException
      */
-    public static function update($id, $request=array()){
+    public static function update($id, $request=array()): array {
         if(!session::isAdmin()){ throw new ForbiddenException('Only administrators can install updates'); }
         $request['username']=session::getUsername();
         $request['gmtDate']=gmdate('d M y H:i:s');
@@ -82,7 +82,7 @@ class updater extends baseobject {
      * @throws ForbiddenException
      * @throws ServerException
      */
-    public static function create($request=array()){
+    public static function create($request=array()): array {
         if(!session::isAdmin()){ throw new ForbiddenException('Only administrators can install updates'); }
         return static::doUpdate($request);
     }
@@ -116,7 +116,7 @@ class updater extends baseobject {
      * @throws ServerException
      * @throws Exception
      */
-    private static function doUpdate($request){
+    private static function doUpdate($request): array {
         set_time_limit(0);
         $newVersion=$request['newversion'];
         if(!Log::isInited()){
@@ -144,7 +144,7 @@ class updater extends baseobject {
             if(!$newVersion || !preg_match('/^(\d+\.)?(\d+\.)?(\d+)$/', $newVersion)){
                 throw new BadRequestException('Bad or missing update version specified');
             }
-            Log::write(Log::LOGLEVEL_INFO, 'Current version: '.@file_get_contents(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/conf/codeversion'));
+            Log::write(Log::LOGLEVEL_INFO, 'Current version: '.config::getCodeVersion());
             Log::write(Log::LOGLEVEL_INFO, 'Updating to version: '.$newVersion);
             
             $updatePath=static::getUpdatePath($newVersion);
@@ -218,8 +218,8 @@ class updater extends baseobject {
             throw $e;
             //throw new ServerException($e->getMessage());
         } finally {
-            $codeVersion=@file_get_contents(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/conf/codeversion');
-            $databaseVersion=config::get('core_icebearversion');
+            $codeVersion=config::getCodeVersion();
+            $databaseVersion=config::getDatabaseVersion();
             Log::write(Log::LOGLEVEL_INFO, 'Code version: '.$codeVersion);
             Log::write(Log::LOGLEVEL_INFO, 'Database version: '.$databaseVersion);
             if(static::$completed){
@@ -251,13 +251,14 @@ class updater extends baseobject {
                 throw new ServerException('Update failed. Possible PHP error.');
             }
         }
+		return []; //Should be unreachable but IDE insists.
     }
 
     /**
      * @throws ServerException
      * @throws Exception
      */
-    private static function backUpCodeAndDatabase(){
+    private static function backUpCodeAndDatabase(): void {
         Log::write(Log::LOGLEVEL_INFO, 'Attempting copy of database and IceBear www directory into file store...');
         Log::write(Log::LOGLEVEL_INFO, '...(should work regardless of whether IceBear backups are configured)...');
         
@@ -274,17 +275,17 @@ class updater extends baseobject {
         }
         
         //get db connection details from conf/config.ini
-        $dbDetails=parse_ini_file(rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/conf/config.ini');
+        $dbDetails=parse_ini_file(config::getWwwRoot().'/conf/config.ini');
         if(!$dbDetails){
             throw new Exception('Could not read database connection details from config file');
         }
-        $db = new PDO('mysql:host='.$dbDetails['dbHost'].';dbname='.$dbDetails['dbName'], $dbDetails['dbUser'], $dbDetails['dbPass'], array(PDO::MYSQL_ATTR_FOUND_ROWS => true));
-        $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
+//        $db = new PDO('mysql:host='.$dbDetails['dbHost'].';dbname='.$dbDetails['dbName'], $dbDetails['dbUser'], $dbDetails['dbPass'], array(PDO::MYSQL_ATTR_FOUND_ROWS => true));
+//        $db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
         
         //Write lock file to prevent importers beginning during dump
         Log::write(Log::LOGLEVEL_DEBUG, 'Writing lock file before database backup...');
-        $imageStoreRoot=rtrim(config::get('core_imagestore'),'/\\').'/';
-        if(!@touch($imageStoreRoot.'dbbackuprunning')){
+        $fileStoreRoot=rtrim(config::get('core_filestore'),'/\\').'/';
+        if(!@touch($fileStoreRoot.'dbbackuprunning')){
             throw new Exception("Could not create lock file before database backup");
         }
         Log::write(Log::LOGLEVEL_DEBUG, '...Done.');
@@ -292,13 +293,13 @@ class updater extends baseobject {
         //Dump the database to .sql file
         Log::write(Log::LOGLEVEL_DEBUG, 'Commencing database dump...');
         $sqlDumpFile=static::$updatePath.'beforeUpdate/icebear.sql';
-        $cmd='mysqldump '.$dbDetails['dbName'].' --password=\''.$dbDetails['dbPass'].'\' --user=\''.$dbDetails['dbUser'].'\' --single-transaction >'.$sqlDumpFile;
+        $cmd='mysqldump '.$dbDetails['dbName'].' --password=\''.$dbDetails['dbPass'].'\' --user=\''.$dbDetails['dbUser'].'\' --no-tablespaces --single-transaction >'.$sqlDumpFile;
         exec($cmd,$output);
         Log::write(Log::LOGLEVEL_DEBUG, '...Database dump done.');
         
         //remove lock file
         Log::write(Log::LOGLEVEL_DEBUG, 'Removing lock file after database backup...');
-        if(!@unlink($imageStoreRoot.'dbbackuprunning')){
+        if(!@unlink($fileStoreRoot.'dbbackuprunning')){
             throw new Exception("Could not remove lock file after database backup");
         }
         Log::write(Log::LOGLEVEL_DEBUG, '...Done.');
@@ -321,11 +322,11 @@ class updater extends baseobject {
         }
         
         Log::write(Log::LOGLEVEL_INFO, 'Backing up existing IceBear code...');
-        Log::write(Log::LOGLEVEL_DEBUG, 'From: '.$_SERVER['DOCUMENT_ROOT']);
+        Log::write(Log::LOGLEVEL_DEBUG, 'From: '.config::getWwwRoot());
         Log::write(Log::LOGLEVEL_DEBUG, 'To: '.static::$updatePath.'beforeUpdate/www');
         Log::write(Log::LOGLEVEL_DEBUG, 'Excluding file store: '.config::get('core_filestore'));
         Log::write(Log::LOGLEVEL_DEBUG, 'Excluding image store: '.config::get('core_imagestore'));
-        static::recursiveCopy($_SERVER['DOCUMENT_ROOT'], static::$updatePath.'beforeUpdate/www',array(
+        static::recursiveCopy(config::getWwwRoot(), static::$updatePath.'beforeUpdate/www',array(
             //If the file or image stores are under www-root, this could go infinite. Do not want.
             config::get('core_imagestore'),
             config::get('core_filestore'),
@@ -335,12 +336,12 @@ class updater extends baseobject {
         Log::write(Log::LOGLEVEL_INFO, '...IceBear database and www directory copied.');
     }
 
-    /**
-     * @param $newVersion
-     * @throws BadRequestException
-     * @throws ServerException
-     */
-    private static function downloadUpdate($newVersion){
+	/**
+	 * @param string $newVersion
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+    private static function downloadUpdate(string $newVersion): void {
         Log::write(Log::LOGLEVEL_INFO, 'Beginning to download and store update file');
         $updatesBaseUri=config::get('update_baseuri');
         if(!$updatesBaseUri){
@@ -386,11 +387,11 @@ class updater extends baseobject {
         Log::write(Log::LOGLEVEL_INFO, 'Update downloaded successfully.');
     }
 
-    /**
-     * @param $newVersion
-     * @throws ServerException
-     */
-    private static function unpackUpdate($newVersion){
+	/**
+	 * @param string $newVersion
+	 * @throws ServerException
+	 */
+    private static function unpackUpdate(string $newVersion): void {
         Log::write(Log::LOGLEVEL_INFO, 'Beginning to unpack update file');
         $updateFilename='icebear_v'.str_replace('.','_',$newVersion).'.tar';
         if(file_exists(static::$updatePath.'unpacked')){
@@ -419,7 +420,7 @@ class updater extends baseobject {
      * @return bool
      * @throws ServerException
      */
-    private static function wasUpdaterModified(){
+    private static function wasUpdaterModified(): bool {
         Log::write(Log::LOGLEVEL_DEBUG, 'In wasUpdaterModified');
         $currentFile=__FILE__;
         $newFile=static::$updatePath.'unpacked/classes/core/updater.class.php';
@@ -450,13 +451,13 @@ class updater extends baseobject {
      * needed to match the old.
      * @throws ServerException
      */
-    private static function matchSslRedirectStatusInHtaccess(){
+    private static function matchSslRedirectStatusInHtaccess(): void {
         Log::write(Log::LOGLEVEL_INFO, 'Checking SSL redirect rules in old and new .htaccess files...');
         $rules=array(
             'RewriteCond %{HTTPS} !=on',
             'RewriteRule ^ https://%{HTTP_HOST}%{REQUEST_URI} [L,R=301]'
         );
-        $oldHtaccessPath=rtrim($_SERVER['DOCUMENT_ROOT'],'/').'/.htaccess';
+        $oldHtaccessPath=config::getWwwRoot().'/.htaccess';
         $newHtaccessPath=rtrim(static::$updatePath,'/').'/unpacked/.htaccess';
         $oldHtaccess=@file($oldHtaccessPath);
         if(!$oldHtaccess){
@@ -509,7 +510,7 @@ class updater extends baseobject {
     /**
      * @throws ServerException
      */
-    private static function copyUnpackedFilesIntoWebRoot(){
+    private static function copyUnpackedFilesIntoWebRoot(): void {
         Log::write(Log::LOGLEVEL_INFO, 'Copying new IceBear files into web directory...');
         $excludeFullPaths=array();
         foreach(static::$preservePaths as $p){
@@ -517,14 +518,14 @@ class updater extends baseobject {
             $excludeFullPaths[]=static::$updatePath.'unpacked/'.ltrim($p,'/');
         }
         static::$codeTainted=true;
-        static::recursiveCopy(static::$updatePath.'unpacked', $_SERVER['DOCUMENT_ROOT'], $excludeFullPaths);
+        static::recursiveCopy(static::$updatePath.'unpacked', config::getWwwRoot(), $excludeFullPaths);
         Log::write(Log::LOGLEVEL_INFO, 'Files copied successfully.');
     }
 
     /**
      * @throws ServerException
      */
-    private static function restoreOriginalCode(){
+    private static function restoreOriginalCode(): void {
         Log::write(Log::LOGLEVEL_DEBUG, 'In restoreOriginalCode');
         if(!static::$codeTainted){
             Log::write(Log::LOGLEVEL_INFO, 'IceBear code was not modified.');
@@ -536,7 +537,7 @@ class updater extends baseobject {
             //Turn relative paths in array into absolutes
             $excludeFullPaths[]=static::$updatePath.'beforeUpdate/www/'.ltrim($p,'/');
         }
-        static::recursiveCopy(static::$updatePath.'beforeUpdate/www', $_SERVER['DOCUMENT_ROOT'], $excludeFullPaths);
+        static::recursiveCopy(static::$updatePath.'beforeUpdate/www', config::getWwwRoot(), $excludeFullPaths);
         static::$codeRestoreAttempted=true;
         Log::write(Log::LOGLEVEL_INFO, 'Attempted restore of original IceBear files.');
         Log::write(Log::LOGLEVEL_DEBUG, 'restoreOriginalCode finished');
@@ -548,9 +549,8 @@ class updater extends baseobject {
      * @throws BadRequestException
      * @throws ForbiddenException
      * @throws ServerException if codeversion file not found, root empty or does not exist, or no viable update path, or update fails.
-     * @noinspection PhpIncludeInspection
-     */
-    public static function updateDatabase($icebearRoot){
+	 */
+    public static function updateDatabase(string $icebearRoot): void {
         Log::write(Log::LOGLEVEL_DEBUG, 'In updateDatabase, $icebearRoot='.$icebearRoot);
         if(!$icebearRoot){
             throw new ServerException('No IceBear root specified.');
@@ -559,11 +559,8 @@ class updater extends baseobject {
             throw new ServerException('IceBear directory does not exist.');
         }
         $icebearRoot=rtrim($icebearRoot,'/').'/';
-        if(!file_exists($icebearRoot.'conf/codeversion')){
-            throw new ServerException('IceBear directory does not contain codeversion file.');
-        }
         $databaseVersion=static::getDatabaseVersion();
-        $codeVersion=static::getCodeVersion($icebearRoot);
+        $codeVersion=static::getCodeVersion();
         Log::write(Log::LOGLEVEL_DEBUG, 'Code version: '.$codeVersion);
         Log::write(Log::LOGLEVEL_DEBUG, 'Database version: '.$databaseVersion);
         if(!file_exists($icebearRoot.'upgrade')){
@@ -573,7 +570,12 @@ class updater extends baseobject {
         //and if no clear path exists we should stop now.
         Log::write(Log::LOGLEVEL_INFO, 'Calculating database update plan...');
         $updatePlan=array();
-        $paths=array_diff(scandir($icebearRoot.'upgrade'), array('..', '.'));
+		$paths=scandir($icebearRoot.'upgrade');
+		if($paths){
+			$paths=array_diff($paths, ['..', '.']);
+		} else {
+			$paths=[];
+		}
         $fromVersion=$databaseVersion;
         $toVersion='';
         $found=false;
@@ -582,7 +584,7 @@ class updater extends baseobject {
             foreach($paths as $p){
                 Log::write(Log::LOGLEVEL_DEBUG, 'Found path '.$p);
                 $found=false;
-                if(strpos($p, 'v'.str_replace('.','_',$fromVersion).'-to-v')===0){
+                if(str_starts_with($p, 'v' . str_replace('.', '_', $fromVersion) . '-to-v')){
                     Log::write(Log::LOGLEVEL_DEBUG, 'Path '.$p.'  matches fromVersion, adding to update plan');
                     $found=true;
                     $p=rtrim($p,'/');
@@ -630,8 +632,8 @@ class updater extends baseobject {
         }
         Log::write(Log::LOGLEVEL_INFO, 'Database update is complete.');
         
-        $codeVersion=@file_get_contents($icebearRoot.'conf/codeversion');
-        $databaseVersion=config::get('core_icebearversion');
+        $codeVersion=config::getCodeVersion();
+        $databaseVersion=config::getDatabaseVersion();
         Log::write(Log::LOGLEVEL_INFO, 'Database version: '.$databaseVersion);
         Log::write(Log::LOGLEVEL_INFO, 'Code version: '.$codeVersion);
         if($databaseVersion!=$codeVersion){
@@ -642,17 +644,20 @@ class updater extends baseobject {
 
     /**
      * @throws ServerException
-     * @noinspection PhpIncludeInspection
-     */
-    public static function loadReferenceData(){
+	 */
+    public static function loadReferenceData(): void {
         Log::write(Log::LOGLEVEL_DEBUG, 'In loadReferenceData');
         $refDataRoot=rtrim(static::$updatePath,'/').'/unpacked/upgrade/referenceData/';
         include_once(rtrim(static::$updatePath,'/').'/unpacked/upgrade/includes/ReferenceDataLoader.class.php');
         ReferenceDataLoader::loadReferenceDataForUpgrade($refDataRoot);
         Log::write(Log::LOGLEVEL_DEBUG, 'loadReferenceData complete');
     }
-    
-    private static function parseHomepageBricks(){
+
+	/**
+	 * @throws ServerException
+	 * @throws BadRequestException
+	 */
+	private static function parseHomepageBricks(): void {
         homepagebrick::parseBrickFiles();
     }
 
@@ -661,7 +666,7 @@ class updater extends baseobject {
      * @param string $dir The full path to the directory to be removed.
      * @throws ServerException
      */
-    private static function recursiveRmdir($dir){
+    private static function recursiveRmdir(string $dir): void {
         
         if(is_dir($dir)) {
             Log::write(Log::LOGLEVEL_DEBUG, 'In recursiveRmdir, removing '.$dir.'...');
@@ -689,9 +694,9 @@ class updater extends baseobject {
      * @param array $pathsToExclude The full paths of any files or directories to exclude from the copy.
      * @throws ServerException
      */
-    private static function recursiveCopy($fromDir, $toDir, $pathsToExclude=array('|')){
+    private static function recursiveCopy(string $fromDir, string $toDir, array $pathsToExclude=array('|')): void {
         Log::write(Log::LOGLEVEL_DEBUG, 'Copying '.$fromDir.' to '.$toDir.'...');
-        //Replace forward and back slashes in excluded paths with pipe, to ease comparison later
+        //Replace forward- and backslashes in excluded paths with pipe, to ease comparison later
         //Also verify that source and destination directories actually exist before starting copy.
         //For performance reasons, we only want to do this once, not every time we recurse!
         if($pathsToExclude[count($pathsToExclude)-1]!='|'){
@@ -718,7 +723,7 @@ class updater extends baseobject {
                     continue;
                 }
                 
-                Log::write(Log::LOGLEVEL_DEBUG, '$fromPath='.$fromPath.'');
+                Log::write(Log::LOGLEVEL_DEBUG, '$fromPath='.$fromPath);
                 if(is_dir($fromPath)){
                     Log::write(Log::LOGLEVEL_DEBUG, ' - Is a directory');
                     if(!@file_exists($toPath)){
@@ -739,39 +744,25 @@ class updater extends baseobject {
                 }
             }
         }
-        Log::write(Log::LOGLEVEL_DEBUG, '...copied '.$fromDir.' to '.$toDir.'');
+        Log::write(Log::LOGLEVEL_DEBUG, '...copied '.$fromDir.' to '.$toDir);
         
     }
 
     /**
-     * @return string|null
+     * @return string
      * @throws ServerException
      * @throws BadRequestException
      */
-    private static function getDatabaseVersion(){
-        $databaseVersion=config::get('core_icebearversion');
-        if(empty($databaseVersion)){
-            Log::write(Log::LOGLEVEL_WARN,'Database version not set. Assuming and setting 1.0');
-            database::query(
-                "INSERT INTO config(name,description,type,defaultvalue,value) 
-                    VALUES('core_icebearversion','IceBear database version number','text','','1.0')",
-                array()
-            );
-            $databaseVersion='1.0';
-        }
-        return $databaseVersion;
-    }
+    private static function getDatabaseVersion(): string {
+		return config::getDatabaseVersion();
+	}
 
     /**
-     * @param $icebearRoot
-     * @return false|string
+     * @return string
      * @throws ServerException
      */
-    private static function getCodeVersion($icebearRoot){
-        if(!file_exists($icebearRoot.'conf/codeversion')){
-            throw new ServerException('IceBear directory does not contain codeversion file.');
-        }
-        return @file_get_contents($icebearRoot.'conf/codeversion');
+    private static function getCodeVersion(): string {
+		return config::getCodeVersion();
     }
 
 }
\ No newline at end of file
