@@ -39,10 +39,10 @@ let Fishing={
 		document.getElementById("barcodeform").onsubmit=function(){ Fishing.handleScan(); return false; };
 		document.getElementById("configbutton").addEventListener("click",Fishing.toggleConfig);
 		window.setInterval(Fishing.focusBarcodeField,250);
-		let qs=document.location.search.toQueryParams();
+		let qs=new URLSearchParams(document.location.search.substring(1));
 		let barcodes=[];
-		if(qs["barcodes"]){
-			barcodes=qs["barcodes"].split(",");
+		if(qs.has("barcodes")){
+			barcodes=qs.get("barcodes").split(",");
 		}
 		let previousBarcodes=Fishing.getWorkspaceBarcodesFromCookie();
 		if(previousBarcodes && 0!==previousBarcodes.length){
@@ -95,7 +95,7 @@ let Fishing={
 	 * Fetches all container types ready for unknown container creation
 	 */
 	getContainerTypes:function(){
-		new Ajax.Request('/api/containertype',{
+		new AjaxUtils.Request('/api/containertype',{
 			method:'get',
 			onSuccess:Fishing.getContainerTypes_onSuccess,
 			onFailure:Fishing.getContainerTypes_onFailure
@@ -219,11 +219,11 @@ let Fishing={
 					if(!ci.welldropid && -1===ci["name"].indexOf("dummypin")){
 						barcode=ci["name"];
 					}
-					Fishing.containersOnBench=Fishing.containersOnBench.without(barcode);
+					Fishing.containersOnBench=ui.arrayWithout(Fishing.containersOnBench,barcode);
 					if(ci.childitems){
 						ci.childitems.forEach(function(cci){
 							if(cci["name"] &&""!==cci["name"]){
-								Fishing.containersOnBench=Fishing.containersOnBench.without(cci["name"]);
+								Fishing.containersOnBench=ui.arrayWithout(Fishing.containersOnBench,cci["name"]);
 							}
 						});
 					}
@@ -242,7 +242,7 @@ let Fishing={
 				}
 			}
 		}
-		new Ajax.Request("/api/containercontent/"+undo.id,{
+		new AjaxUtils.Request("/api/containercontent/"+undo.id,{
 			method:'delete',
 			parameters:{ csrfToken:csrfToken, },
 			onSuccess:function(transport){ Fishing._undoItemIntoContainer_onSuccess(transport, barcode); },
@@ -269,11 +269,11 @@ let Fishing={
 			});
 		}
 		if(crystalDiv){
-			crystalDiv.classList.remove("fish_alreadyfished");
+			crystalDiv.classList.remove("xtal_fished");
 			crystalDiv.crystal.isfished=0;
 			crystalDiv.title=null;
 		}
-		new Ajax.Request("/api/crystal/"+crystalId,{
+		new AjaxUtils.Request("/api/crystal/"+crystalId,{
 			method:'patch',
 			parameters:{ csrfToken:csrfToken, isfished:0 }
 		});
@@ -303,7 +303,7 @@ let Fishing={
 				break;
 			}
 		}
-		new Ajax.Request("/api/crystal/"+undo.id,{
+		new AjaxUtils.Request("/api/crystal/"+undo.id,{
 			method:'delete',
 			parameters:{ csrfToken:csrfToken, },
 			onSuccess:function(transport){ Fishing._undoCreateCrystal_onSuccess(transport, wd); },
@@ -439,22 +439,22 @@ let Fishing={
 			}
 		});
 		if(conflict){ return false; }
-		new Ajax.Request('/api/plate/name/'+barcode,{
+		new AjaxUtils.Request('/api/plate/name/'+barcode,{
 			method:"get",
 			onSuccess:function(transport){
-				if(!transport.responseJSON || !transport.responseJSON.rows){
+				if(!transport.responseJSON || !transport.responseJSON.id){
 					return AjaxUtils.checkResponse(transport);
 				}
-				Fishing.renderPlate(transport.responseJSON.rows[0]);
+				Fishing.renderPlate(transport.responseJSON);
 			},
 			onFailure:function(){ 
-				new Ajax.Request('/api/container/name/'+barcode,{
+				new AjaxUtils.Request('/api/container/name/'+barcode,{
 					method:"get",
 					onSuccess:function(transport){
-						if(!transport.responseJSON || !transport.responseJSON.rows){
+						if(!transport.responseJSON || !transport.responseJSON.id){
 							return AjaxUtils.checkResponse(transport);
 						}
-						let container=transport.responseJSON.rows[0];
+						let container=transport.responseJSON;
 						Fishing.renderContainer(container);
 					},
 					onFailure:function(){
@@ -474,7 +474,7 @@ let Fishing={
 		Fishing.plateDropsRetrieved=false;
 		Fishing.plateScoresRetrieved=false;
 		Fishing.plateCrystalsRetrieved=false;
-		new Ajax.Request('/api/plate/'+plate.id+'/welldrop',{
+		new AjaxUtils.Request('/api/plate/'+plate.id+'/welldrop',{
 			method:"get",
 			onSuccess:function(transport){
 				Fishing.plate.welldrops=transport.responseJSON.rows;
@@ -482,12 +482,12 @@ let Fishing={
 			}
 		});
 		if(""!==plate.screenid){
-			new Ajax.Request('/api/screen/'+plate.screenid+'/screencondition?pagenum=1&pagesize=10000',{
+			new AjaxUtils.Request('/api/screen/'+plate.screenid+'/screencondition?all=1',{
 				method:"get",
 				onSuccess:Fishing.attachScreenConditions
 			});
 		}
-		new Ajax.Request('/api/plate/'+plate.id+'/crystal?pagenum=1&pagesize=10000',{
+		new AjaxUtils.Request('/api/plate/'+plate.id+'/crystal?all=1',{
 			method:'get',
 			onSuccess:function(transport){
 				Fishing.plate.crystals=transport.responseJSON.rows;
@@ -498,7 +498,7 @@ let Fishing={
 				Fishing.plateCrystalsRetrieved=true;
 			}
 		});
-		new Ajax.Request('/api/plate/'+plate.id+'/dropbestscore?pagenum=1&pagesize=10000',{
+		new AjaxUtils.Request('/api/plate/'+plate.id+'/dropbestscore?all=1',{
 			method:'get',
 			onSuccess:function(transport){
 				Fishing.plate.scores=transport.responseJSON.rows;
@@ -525,11 +525,12 @@ let Fishing={
 		}
 		let header=box.querySelector("h2");
 		let boxbody=box.querySelector(".boxbody");
-		
+
 		//whole boxbody goes up under header when platewidget is called. Correct for it.
-		boxbody.absolutize();
-		boxbody.style.top=header.getHeight()+"px";
-		
+		//20250220 - No evidence of this behaviour with below lines removed. Leaving in place for now but commented out.
+		// boxbody.absolutize();
+		// boxbody.style.top=header.offsetHeight+"px";
+
 		boxbody.innerHTML="";
 		let allWellsHaveConstructs=true;
 		Fishing.plate.welldrops.forEach(function(wd){
@@ -545,7 +546,7 @@ let Fishing={
 			wellDrops:Fishing.plate.welldrops,
 			dropPickerPosition:"left"
 		}, boxbody);
-		pw.plate=pw.plateType;
+		pw.plate=pw["plateType"];
 		pw.dataset.itemname=plate.name;
 		header.innerHTML="Plate: "+plate.name;
 		let drops=pw.querySelector(".pw_plate").querySelectorAll(".pw_drop");
@@ -586,11 +587,11 @@ let Fishing={
 		xtaldrag.dataset.welldropid=dropDiv.dataset.welldropid;
 		dropDiv.appendChild(xtaldrag);
 		if(dropDiv.score){
-			dropDiv.style.backgroundColor="#"+dropDiv.score.bestscorecolor;
+			dropDiv.style.backgroundColor="#"+dropDiv.score["bestscorecolor"];
 		}
 		let numCrystals=dropDiv.crystals.length;
 		if(numCrystals>0){
-			xtalcount.querySelector("span").innerHTML=numCrystals;
+			xtalcount.querySelector("span").innerHTML=numCrystals+"";
 			xtalcount.classList.remove("empty");
 		}
 		let hasConstruct=(0!==1*dropDiv.wellDrop.constructid);
@@ -629,7 +630,7 @@ let Fishing={
 	 * @param bypassDupeCheck If set, don't check whether this container is already on the bench (e.g., after nesting existing containers)
 	 */
 	renderContainer:function(container, bypassDupeCheck){
-		new Ajax.Request('/api/container/'+container.id+'/fullcontent',{
+		new AjaxUtils.Request('/api/container/'+container.id+'/fullcontent',{
 			method:'get',
 			onSuccess:function(transport){
 				let containers=transport.responseJSON.rows;
@@ -720,7 +721,7 @@ let Fishing={
 		let contentCount=0;
 		let cc=elem.querySelector(".fish_contentcount");
 		elem.container.childitems.forEach(function(c){
-			if(c.childitems || c.welldropid || c.crystalhidden){
+			if(c["childitems"] || c["welldropid"] || c["crystalhidden"]){
 				contentCount++;
 			}
 		});
@@ -792,7 +793,7 @@ let Fishing={
 			return false;
 		}
 		btn.closest("label").classList.add("updating");
-		new Ajax.Request("/api/containercontent/"+pin.childitems[1].containercontentid,{
+		new AjaxUtils.Request("/api/containercontent/"+pin.childitems[1].containercontentid,{
 			method:"delete",
 			parameters:{
 				csrfToken:csrfToken
@@ -819,16 +820,15 @@ let Fishing={
 	addUnpackButtons:function(){
 
 		let mb=document.getElementById("modalBox");
-
 		let unpackIcon='<img style="min-height:2.5em; cursor:pointer" '+
-		'src="/images/icons/'+skin["bodyicontheme"]+'/unpack.gif" '+
+		'src="/images/icons/'+skin["bodyIconTheme"]+'icons/unpack.gif" '+
 		'title="Unpack and place onto workbench" onclick="Fishing.unpackToBench(this)"  alt=""/>';
 		
 		if(mb.querySelector(".treeitem")){
 			// This is a dewar. Add the unpack icon to the right-hand end of each tree item
 			// which has a puck.
 			unpackIcon='<img style="cursor:pointer; height:2em; position:absolute; top:0.25em;right:0.5em; background-color:white;border-radius:3px" '+
-			'src="/images/icons/'+skin["boxheadericontheme"]+'/unpack.gif" '+
+			'src="/images/icons/'+skin["treeHeaderIconTheme"]+'icons/unpack.gif" '+
 			'title="Unpack and place onto workbench" onclick="Fishing.unpackToBench(this)"  alt=""/>';
 			
 			mb.querySelectorAll(".treeitem").forEach(function(elem){
@@ -839,7 +839,7 @@ let Fishing={
 				}
 			});
 		} else {
-			// This is a puck. Add a extra cell to each table row, then add the unpack icon to
+			// This is a puck. Add an extra cell to each table row, then add the unpack icon to
 			// the new cell for any rows that have a pin.
 			mb.querySelectorAll("tr").forEach(function(tr){
 				tr.innerHTML+="<td></td>";
@@ -858,7 +858,7 @@ let Fishing={
 		let item=clickedIcon.closest("tr, .treeitem");
 		if(item.rowData){
 			item.classList.add("updating");
-			new Ajax.Request('/api/containercontent/'+item.rowData.containercontentid,{
+			new AjaxUtils.Request('/api/containercontent/'+item.rowData.containercontentid,{
 				method:'delete',
 				parameters:{ csrfToken:csrfToken },
 				onSuccess:function(){ Fishing.unpackPinToBench_onSuccess(item); },
@@ -866,7 +866,7 @@ let Fishing={
 			})
 		} else if(item.record){
 			item.querySelector(".treehead").classList.add("updating");
-			new Ajax.Request('/api/containercontent/'+item.record.containercontentid,{
+			new AjaxUtils.Request('/api/containercontent/'+item.record.containercontentid,{
 				method:'delete',
 				parameters:{ csrfToken:csrfToken },
 				onSuccess:function(){ Fishing.unpackPuckToBench_onSuccess(item); },
@@ -969,9 +969,9 @@ let Fishing={
 			if(0===pins.length||0===slots.length){ return; }
 			pins.forEach(function(pin){
 				pin.style.width="12em";
-				let name=pin.closest("tr").rowData.name.trim();
+				let name=pin.closest("tr")["rowData"].name.trim();
 				if(""===name || 0===name.indexOf("dummypin")){ name="(no barcode)"; }
-				pin.innerHTML='<div class="fish_pininpuck" id="pininpuck'+pin.closest("tr").rowData.id+'">'+name+'</div>';
+				pin.innerHTML='<div class="fish_pininpuck" id="pininpuck'+pin.closest("tr")["rowData"].id+'">'+name+'</div>';
 			});
 			slots.forEach(function(slot){
 				slot.innerHTML="";
@@ -1045,9 +1045,9 @@ let Fishing={
 		if(!slot || !pin){ return false; }
 		slot.classList.remove("fish_updated","fish_activedroppable");
 		slot.classList.add("updating");
-		let pinData=pin.closest("tr").rowData;
+		let pinData=pin.closest("tr")["rowData"];
 		let slotData=slot.closest("tr").rowData;
-		new Ajax.Request('/api/containercontent/'+pinData.containercontentid,{
+		new AjaxUtils.Request('/api/containercontent/'+pinData.containercontentid,{
 			method:'patch',
 			parameters:{
 				csrfToken:csrfToken,
@@ -1095,7 +1095,7 @@ let Fishing={
 			let categoryDiv=document.getElementById("fish_ct_"+catName);
 			if(!categoryDiv){ return; } //Not interested in this containercategory
 			let canCreate=true;
-			if(!ct.userscancreate && !isAdmin && !isTechnician){
+			if(!ct["userscancreate"] && !isAdmin && !isTechnician){
 				canCreate=false;
 				if(categoryDiv.querySelector(".fish_containertype")){
 					//already wrote the "You can't create whatever(s)" div
@@ -1166,14 +1166,26 @@ let Fishing={
 		}
 	},
 	findAndAttachImages:function(dropDiv){
-		new Ajax.Request('/api/welldrop/'+dropDiv.dataset.welldropid+'/timecourseimage',{
+		new AjaxUtils.Request('/api/welldrop/'+dropDiv.dataset.welldropid+'/timecourseimage',{
 			method:'get',
 			onSuccess:function(transport){ Fishing.findAndAttachImages_onSuccess(transport,dropDiv) },
 			onFailure:function(transport){ Fishing.findAndAttachImages_onFailure(transport,dropDiv) },
 		});
 	},
 	findAndAttachImages_onSuccess:function(transport,dropDiv){
-		dropDiv.dropImages=transport.responseJSON.rows;
+		let images=[];
+		let rows=transport.responseJSON.rows;
+		rows.forEach(function (img){
+			if("visible"===img["lighttype"].toLowerCase()){
+				images.push(img);
+			}
+		})
+		if(!images.length){
+			//No visible images. Just use the most recent (which is probably UV).
+			images.push(rows[rows.length-1]);
+		}
+		dropDiv["dropImages"]=images;
+		dropDiv["imageIndex"]=images.length-1;
 		Fishing.renderImageAndCrosshairs(dropDiv);
 	},
 	findAndAttachImages_onFailure:function(transport){
@@ -1181,26 +1193,21 @@ let Fishing={
 			AjaxUtils.checkResponse(transport);
 			return false;
 		}
+		alert("There are no images for this drop. Fish by dragging directly from the plate overview to the pin or puck.");
 	},
 	
 	renderImageAndCrosshairs:function(dropDiv){
 		let well=dropDiv.closest(".pw_well");
 		let condition=well.dataset.condition;
 		let box=document.getElementById("platebox");
+		box.dropDiv=dropDiv;
 		let boxbody=box.querySelector(".boxbody");
 		let header=box.querySelector("h2");
 		header.oldInnerHTML=header.innerHTML;
 		header.innerHTML='<img src="/skins/default/images/icons/no.gif" class="closeicon" onclick="Fishing.closeImageAndCrosshairs()" alt="">' + header.oldInnerHTML +
 			" well " + Plate.getWellName(dropDiv.dataset.row, dropDiv.dataset.col, dropDiv.dataset.dropnumber);
 		let numImages=dropDiv.dropImages.length;
-		//Default to last image, then try to find last visible
 		let dropImage=dropDiv.dropImages[numImages-1];
-		for(let i=numImages;i>0;i--){
-			if("visible"===dropDiv.dropImages[i-1]["lighttype"].toLowerCase()){
-				dropImage=dropDiv.dropImages[i-1];
-				break;
-			}
-		}
 		let imgDiv=document.createElement("div");
 		imgDiv.classList.add("fish_dropimage");
 		if(condition){
@@ -1214,11 +1221,12 @@ let Fishing={
 		imgDiv.appendChild(img);
 		boxbody.appendChild(imgDiv);
 		boxbody.style.borderRadius=0;
-		imgDiv.pixelScale=img.getHeight()/dropImage["pixelheight"];
+		imgDiv.pixelScale=img.offsetHeight/dropImage["pixelheight"];
 		dropDiv.crystals.forEach(function(xtal){
 			Fishing.drawCrosshair(xtal);
 		});
 		Fishing.mapDragDropElements();
+		Fishing.renderImageTimeControls(dropDiv);
 
 		if(condition){
 			let condDiv=document.createElement("div");
@@ -1229,45 +1237,138 @@ let Fishing={
 
 		window.onresize=function(){ 
 			Fishing.closeImageAndCrosshairs();
-			Fishing.renderImageAndCrosshairs(dropDiv) 
+			Fishing.renderImageAndCrosshairs(dropDiv);
 		};
-
-		/*
-		 * TODO 
-		 * Events for kb nav to other drops
-		 * Nav in box header
-		 * ....
-		 */
-		
+		Fishing.waitForCrosshairs(dropDiv.crystals.length);
 	},
-	drawCrosshair:function(xtal){
-		let imgDiv=document.getElementById("platebox").querySelector(".fish_dropimage");
-		let scale=imgDiv.pixelScale;
-		let xh=document.createElement("div");
-		xh.classList.add("fish_crosshair");
-		let xhh=document.createElement("div");
-		xhh.classList.add("fish_xhhori");
-		let xhv=document.createElement("div");
-		xhv.classList.add("fish_xhvert");
-		let xhn=document.createElement("div");
-		xhn.classList.add("fish_xhnum");
-		xh.appendChild(xhh);
-		xh.appendChild(xhv);
-		xh.appendChild(xhn);
-		imgDiv.appendChild(xh);
-		xhn.innerHTML=xtal.numberindrop;
-		xh.style.left=((parseInt(xtal.pixelx)*scale)-(xh.getWidth()/2))+"px";
-		xh.style.top =((parseInt(xtal.pixely)*scale)-(xh.getWidth()/2))+"px";
-		xh.crystal=xtal;
-		xh.id="crystal"+xtal.id;
-		if(parseInt(xtal.isfished)){
-			xh.classList.add("fish_alreadyfished"); 
-			xh.title="Crystal has already been fished";
+
+	waitForCrosshairs:function (crystalCount){
+		if(!crystalCount){ return; }
+		let rendered=document.getElementById("fish_currentimage").closest("div").querySelectorAll(".xtal_crosshair");
+		if(rendered.length!==crystalCount){
+			window.setTimeout(Fishing.waitForCrosshairs,250, crystalCount);
+			return;
 		}
+		Fishing.mapDragDropElements();
+	},
+
+	renderImageTimeControls:function (dropDiv){
+		let boxHeader=dropDiv.closest(".box").querySelector("h2");
+		let wrapper=boxHeader.querySelector(".timeControls");
+		if(wrapper){ wrapper.remove(); }
+		let images=dropDiv["dropImages"];
+		if(!images || 2>images.length){
+			return;
+		}
+		wrapper=document.createElement("span");
+		wrapper.classList.add("timeControls");
+		wrapper.style.position="absolute";
+		wrapper.style.right="4em";
+		wrapper.style.top="0.2em";
+		boxHeader.insertAdjacentElement("beforeend", wrapper);
+		let backArrow=document.createElement("img");
+		backArrow.addEventListener("click",Fishing.setPreviousImage);
+		backArrow.style.filter="invert(1)";
+		backArrow.src="/images/icons/darkicons/leftarrow.png";
+		backArrow.id="imageNavForward";
+		backArrow.style.height="1.5em"
+		backArrow.title="Older image";
+		wrapper.appendChild(backArrow);
+		let clock=document.createElement("img");
+		clock.style.filter="invert(1)";
+		clock.style.height="1.5em";
+		clock.style.margin="0 0.25em";
+		clock.src="/images/icons/darkicons/clock.png";
+		clock.id="imageNavForward";
+		clock.title="Move back and forward in time";
+		wrapper.appendChild(clock);
+		let forwardArrow=document.createElement("img");
+		forwardArrow.addEventListener("click",Fishing.setNextImage);
+		forwardArrow.src="/images/icons/darkicons/rightarrow.png";
+		forwardArrow.style.filter="invert(1)";
+		forwardArrow.id="imageNavForward";
+		forwardArrow.style.height="1.5em"
+		forwardArrow.title="Newer image";
+		wrapper.appendChild(forwardArrow);
+		let imageIndex=dropDiv["imageIndex"];
+		if(imageIndex>0){
+			//attach event to back arrow, cursor:pointer
+			backArrow.style.cursor="pointer";
+			backArrow.style.opacity="1";
+		} else {
+			backArrow.style.cursor="default";
+			backArrow.style.opacity="0.4";
+		}
+		if(imageIndex<images.length-1){
+			//attach event to forward arrow, cursor:pointer
+			forwardArrow.style.cursor="pointer";
+			forwardArrow.style.opacity="1";
+		} else {
+			forwardArrow.style.cursor="default";
+			forwardArrow.style.opacity="0.4";
+		}
+	},
+	setPreviousImage:function (){
+		let querySelector="tr.row"+Fishing.currentImageRow+" td.col"+Fishing.currentImageCol+" .drop"+Fishing.currentImageDrop;
+		let dropDiv=document.getElementById("platebox").querySelector(querySelector);
+		let imageIndex=dropDiv["imageIndex"];
+		if(imageIndex<=0){ return false; }
+		Fishing.setImage(dropDiv,imageIndex-1);
+	},
+	setNextImage:function (){
+		let querySelector="tr.row"+Fishing.currentImageRow+" td.col"+Fishing.currentImageCol+" .drop"+Fishing.currentImageDrop;
+		let dropDiv=document.getElementById("platebox").querySelector(querySelector);
+		let imageIndex=dropDiv["imageIndex"];
+		let images=dropDiv["dropImages"];
+		if(imageIndex>=images.length-1){ return false; }
+		Fishing.setImage(dropDiv,imageIndex+1);
+	},
+	setImage:function (dropDiv,imageIndex){
+		dropDiv["imageIndex"]=imageIndex;
+		let dropImage=dropDiv["dropImages"][imageIndex];
+		let img=document.getElementById("fish_currentimage");
+		let imgDiv=img.closest("div");
+		imgDiv.pixelScale=img.offsetHeight/dropImage["pixelheight"];
+		img.src=dropImage["fullimageurl"];
+		img.dropImage=dropImage;
+		imgDiv.querySelectorAll(".xtal_crosshair").forEach(function (crosshair){
+			crosshair.remove();
+		});
+		dropDiv.crystals.forEach(function(xtal){
+			Fishing.drawCrosshair(xtal);
+		});
+		Fishing.mapDragDropElements();
+		Fishing.renderImageTimeControls(dropDiv);
+	},
+
+	drawCrosshair:function(xtal){
+		let img=document.getElementById("fish_currentimage");
+		if(!img.naturalHeight){
+			window.setTimeout(Fishing.drawCrosshair, 250, xtal);
+			return;
+		}
+		let xh=Crystal.drawCrosshair(xtal, img);
 		let drag=document.createElement("div");
 		drag.classList.add("fish_xtaldrag");
 		drag.crystal=xtal;
 		xh.appendChild(drag);
+		if(xh.classList.contains("xtal_crosshairfromdifferentimage")){
+			let querySelector="tr.row"+Fishing.currentImageRow+" td.col"+Fishing.currentImageCol+" .drop"+Fishing.currentImageDrop;
+			let dropDiv=document.getElementById("platebox").querySelector(querySelector);
+			let images=dropDiv["dropImages"];
+			let index=-1;
+			let found=false;
+			images.forEach(function (img){
+				if(found){ return; }
+				index++;
+				if(img["id"]===xtal["dropimageid"]){
+					found=true;
+					xh.addEventListener("click", function (){
+						Fishing.setImage(dropDiv, index);
+					});
+				}
+			});
+		}
 	},
 	
 	closeImageAndCrosshairs:function(){
@@ -1297,16 +1398,16 @@ let Fishing={
 	 */
 	markCrystalOnImage:function(evt){
 		let img=evt.target;
-		//Crystal number should be highest number in the drop, +1.
+		//Crystal number should be the highest number in the drop, +1.
 		let crystalNumber=0;
 		let currentImage=document.getElementById("fish_currentimage");
-		currentImage.closest(".fish_dropimage").querySelectorAll(".fish_crosshair").forEach(function(c){
+		currentImage.closest(".fish_dropimage").querySelectorAll(".xtal_crosshair").forEach(function(c){
 			crystalNumber=Math.max(crystalNumber,c.crystal.numberindrop);
 		});
 		crystalNumber++;
 		//Coordinates
 		let offset=ui.cumulativeOffset(currentImage);
-		let scale=currentImage.getHeight() / img.dropImage["pixelheight"];
+		let scale=currentImage.offsetHeight / img.dropImage["pixelheight"];
 		let crystalX=Math.round((evt.clientX-offset.left)/scale);
 		let crystalY=Math.round((evt.clientY-offset.top)/scale);
 		let pb=document.getElementById("platebox");
@@ -1324,7 +1425,7 @@ let Fishing={
 	},
 	
 	createCrystal:function(crystal,container){
-		new Ajax.Request('/api/crystal',{
+		new AjaxUtils.Request('/api/crystal',{
 			method:'post',
 			parameters:{
 				csrfToken:csrfToken,
@@ -1383,8 +1484,8 @@ let Fishing={
 	mapDragDropElements:function(){
 		let dragIcons=["crystal","pin","puck","dewar"];
 		dragIcons.forEach(function(dragIcon){
-			img=document.createElement("img");
-			img.src="/images/icons/"+skin["bodyicontheme"]+"/fishdrag_"+dragIcon+".png";
+			let img = document.createElement("img");
+			img.src="/images/icons/"+skin["bodyIconTheme"]+"icons/fishdrag_"+dragIcon+".png";
 			Fishing.dragImages[dragIcon]=img;
 		});
 		window.setTimeout(function(){
@@ -1456,14 +1557,14 @@ let Fishing={
 	
 	dragCrystalStart:function(evt){
 		let dragged=evt.target;
-		if(dragged.closest(".fish_alreadyfished")){
+		if(dragged.closest(".xtal_fished")){
 			alert("This crystal has already been fished.");
 			return false;
 		}
 		Fishing.resetCurrentFish();
 		dragged.classList.add("fish_draggingcrystal");
 		evt.dataTransfer.dropEffect="move";
-		evt.dataTransfer.setData("text", dragged.closest(".fish_crosshair,.pw_drop").id);
+		evt.dataTransfer.setData("text", dragged.closest(".xtal_crosshair,.pw_drop").id);
 		let img=Fishing.dragImages["crystal"];
 		evt.dataTransfer.setDragImage(img,-10,50);
 		dragged.droppables.forEach(function(d){ d.classList.add("fish_droppable"); });
@@ -1572,13 +1673,13 @@ let Fishing={
 		trash.classList.remove("fish_updated");
 		//We preserve the container's contents. Nothing to do there.
 		//All we need to do is update the containers cookie so the container doesn't come back at reload.
-		Fishing.containersOnBench=Fishing.containersOnBench.without(dropped.container.name);
+		Fishing.containersOnBench=ui.arrayWithout(Fishing.containersOnBench,dropped.container.name);
 		if(dropped.container.childitems){
 			dropped.container.childitems.forEach(function(ci){
-				Fishing.containersOnBench=Fishing.containersOnBench.without(ci.name);
+				Fishing.containersOnBench=ui.arrayWithout(Fishing.containersOnBench,ci.name);
 				if(ci.childitems){
 					ci.childitems.forEach(function(cci){
-						Fishing.containersOnBench=Fishing.containersOnBench.without(cci.name);
+						Fishing.containersOnBench=ui.arrayWithout(Fishing.containersOnBench,cci.name);
 					});
 				}
 			});
@@ -1662,7 +1763,7 @@ let Fishing={
 			//drag-drop setup should prevent. No other handling needed.
 			return false;
 		}
-		new Ajax.Request('/api/containercontent',{
+		new AjaxUtils.Request('/api/containercontent',{
 			method:'post',
 			parameters:{
 				csrfToken:csrfToken,
@@ -1687,7 +1788,7 @@ let Fishing={
 			//crystal crosshair was dragged
 			let crystalCrosshair=document.getElementById("crystal"+contained.id);
 			crystalCrosshair.crystal.isfished=1;
-			crystalCrosshair.classList.add("fish_alreadyfished");
+			crystalCrosshair.classList.add("xtal_fished");
 			Fishing.renderPlateWidgetDrop(document.getElementById("welldrop"+contained.welldropid));
 		} else if(document.getElementById("welldrop"+contained.id)){
 			//well drop was dragged, should have created crystal
@@ -1741,7 +1842,7 @@ let Fishing={
 	saveNoteOnLastFishedCrystal:function(noteText, successCallback){
 		let crystal=Fishing.currentFish.crystal;
 		if(""===noteText){ return false; }
-		new Ajax.Request('/api/note',{
+		new AjaxUtils.Request('/api/note',{
 			method:'post',
 			parameters:{
 				csrfToken:csrfToken,
@@ -1789,7 +1890,7 @@ let Fishing={
 	    //add to recents
 		Fishing.recentNotes.forEach(function(n){
 			if(n===note){
-				Fishing.recentNotes=Fishing.recentNotes.without(n);
+				Fishing.recentNotes=ui.arrayWithout(Fishing.recentNotes,n);
 			}
 		});
 		Fishing.recentNotes.push(note);
@@ -1809,14 +1910,19 @@ let Fishing={
 		let recentNotesForm=document.getElementById("recentnotesform");
 		recentNotesForm.innerHTML='';
 		let lbl=document.createElement("label");
-		lbl.style="cursor:pointer;text-align:center;font-weight:bold;padding:1em";
+		lbl.style.cursor="pointer";
+		lbl.style.textAlign="center";
+		lbl.style.fontWeight="bold";
+		lbl.style.padding="1em";
 		lbl.innerHTML='Click the note below to use it, or click here to cancel.';
 		lbl.addEventListener("click", Fishing.cancelRecentNotes);
 		recentNotesForm.appendChild(lbl);
 		let notes=Fishing.recentNotes.slice(-1*Fishing.recentNotesToOffer).reverse();
 		notes.forEach(function(n){
 			lbl=document.createElement("label");
-			lbl.style="cursor:pointer;text-align:left;padding:0.5em";
+			lbl.style.cursor="pointer"
+			lbl.style.textAlign="left";
+			lbl.style.padding="0.5em";
 			lbl.addEventListener("click", function(evt){
 				let elem=evt.target;
 				if(elem.closest("label")){ elem=elem.closest("label"); }
