@@ -18,11 +18,14 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function create($request=array()){
-			if(!static::canCreate()){
+	public static function create(array $request=array()): array {
+		if(!static::canCreate()){
 			throw new ForbiddenException('Only administrators and technicians can create containers');
 		}
-		$project=project::getByName(project::SHARED);
+		$project=project::getByName(baseproject::SHARED);
+		if(!$project){
+			throw new NotFoundException('Could not read shared project');
+		}
 		$request['projectid']=$project['id'];
         return parent::createByClassName($request,'containercategory');
 	}
@@ -32,7 +35,7 @@
      * @throws BadRequestException
      * @throws NotFoundException
      */
-	public static function canCreate(){
+	public static function canCreate(): bool {
 		if(session::isAdmin()){ return true; }
 		$grp=usergroup::getByName(usergroup::TECHNICIANS);
 		return usergroup::userisingroup($grp['id']);
