@@ -16,7 +16,7 @@
 			'scoreindex'=>'0 is the lowest score, increase by 1 per score',
 			'hotkey'=>'The hotkey for this score in the drop viewer (0-9 only)'
 	);
-	
+
 	public static $defaultSortOrder='scoreindex ASC';
 
     /**
@@ -27,10 +27,17 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function create($request=array()){
-		$project=project::getByName(project::SHARED);
+	public static function create(array $request=array()): array {
+		$project=project::getByName(baseproject::SHARED);
+		if(!$project){
+			throw new NotFoundException('Could not read shared project');
+		}
 		$request['projectid']=$project['id'];
         return parent::createByClassName($request,'crystalscore');
 	}
-	
+
+	public static function canCreate(): bool {
+		return crystalscoringsystem::canCreate();
+	}
+
 }
\ No newline at end of file
