@@ -8,7 +8,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-    public static function create($request=array()){
+    public static function create(array $request=array()): array {
         if(session::isShipper() && !session::isAdmin()){
             session::becomeAdmin();
         }
@@ -16,10 +16,10 @@
             $ret=parent::create($request);
             session::revertAdmin();
             return $ret;
-        } catch (NotFoundException | BadRequestException | ServerException | ForbiddenException $e){
+        } catch (NotFoundException|ForbiddenException|ServerException|BadRequestException $e){
             session::revertAdmin();
             throw $e;
         }
-    }
+	}
 
 }
\ No newline at end of file
