@@ -15,7 +15,9 @@
 			'inspectionsremaining'=>validator::INTEGER,
 	        'hasconstructs'=>validator::BOOLEAN,
 	        'datedestroyed'=>validator::DATE,
-	);
+            'bestprobability'=>validator::FLOAT,
+            'bestcrystalscoreid'=>validator::INTEGER
+    );
 	
 	protected static $helpTexts=array(
 			'name'=>'A unique name for this plate, typically its barcode',
@@ -52,9 +54,22 @@
 	
 	
     protected static $searchClause='CONCAT(plate.name,": ",plate.description)';
-	
-	protected static function getProjectClause($accessType,$forceSharedProject=false){
-	    return ' AND ((1=1 '.database::getProjectClause($accessType,$forceSharedProject).') OR plate.ownerid='.session::getUserId().')';
+
+	protected static function getProjectClause(string $accessType, bool $forceSharedProject=false): string {
+		if('readOne'===$accessType){
+			$ownPlatesClause='plate.ownerid='.basesession::getUserId();
+		} else if('read'===$accessType){
+			$ownPlatesClause='plate.ownerid='.basesession::getUserId();
+			if(!session::getShowArchivedProjects()){
+				$ownPlatesClause.=' AND project.isarchived=0';
+			}
+		} else {
+			$ownPlatesClause='plate.ownerid='.basesession::getUserId().' AND project.isarchived=0';
+		}
+		if(!('read'===$accessType || (!basesession::getShowArchivedProjects() && 'readOne'===$accessType))){
+			$ownPlatesClause.=' AND project.isarchived=0';
+		}
+		return ' AND ((1=1 '.database::getProjectClause($accessType,$forceSharedProject).') OR ('.$ownPlatesClause.'))';
 	}
 
 
@@ -62,31 +77,27 @@
      * @param int $id
      * @return array|mixed
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      */
-	public static function getById($id){
+	public static function getById($id): ?array {
         $plate=parent::getById($id);
-        if(!$plate){ return $plate; }
-        $plate=static::addImagingStatusByDropPositionNumber($plate);
-        return $plate;
+		if(!$plate){ return $plate; }
+		return static::addImagingStatusByDropPositionNumber($plate);
     }
 
     /**
      * @param string $name
-     * @return array|mixed
+     * @return array|null
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      */
-    public static function getByName($name){
+    public static function getByName($name): ?array {
         $plate=parent::getByName($name);
         if(!$plate){ return $plate; }
-        $plate=static::addImagingStatusByDropPositionNumber($plate);
-        return $plate;
+		return static::addImagingStatusByDropPositionNumber($plate);
     }
 
-    public static function getByProperty($key, $value, $request = array()){
+    public static function getByProperty($key, $value, $request = array()): ?array {
         $plates=parent::getByProperty($key, $value, $request);
         if(!$plates || 1!=$plates['total']){
             return $plates;
@@ -137,13 +148,14 @@
      * @throws ForbiddenException
      * @throws NotFoundException
      * @throws ServerException
-     * @throws NotModifiedException
-     */
-	public static function create($request=array()){
+	 */
+	public static function create(array $request=array()): array {
 		$ret=parent::createByClassName($request,'plate');
+		if(!$ret){ throw new ServerException('Could not create plate'); }
 		$plate=$ret['created'];
 		$plateId=$plate['id'];
 		$plateType=platetype::getById($plate['platetypeid']);
+		if(!$plateType){ throw new ServerException('Could not find plate type with ID '.$plate['platetypeid']); }
 		$numRows=$plateType['rows'];
 		$numCols=$plateType['cols'];
 		$numDrops=$plateType['subs'];
@@ -182,7 +194,7 @@
      * @throws NotModifiedException
      * @throws ServerException
      */
-	public static function update($id, $request=array()){
+	public static function update($id, $request=array()): array {
 		$plate=static::getById($id);
 
 		if(isset($request['datedestroyed'])){
@@ -195,12 +207,12 @@
 		$cols=(int)($plateType['cols']);
 		
 		//Set drop number to update - default to -1 (all drops)
-		$dropNumber= isset($request['selectionDrop']) ? $request['selectionDrop'] : -1;
+		$dropNumber= $request['selectionDrop'] ?? -1;
 		//Set the range of wells to update - defaults are -1 to 10000, which then narrows to smaller of (selection) or (entire plate)
-		$request['selectionTop']=   isset($request['selectionTop'])    ? $request['selectionTop']    : -1;
-		$request['selectionLeft']=  isset($request['selectionLeft'])   ? $request['selectionLeft']   : -1;
-		$request['selectionBottom']=isset($request['selectionBottom']) ? $request['selectionBottom'] : 10000;
-		$request['selectionRight']= isset($request['selectionRight'])  ? $request['selectionRight']  : 10000;
+		$request['selectionTop']= $request['selectionTop'] ?? -1;
+		$request['selectionLeft']= $request['selectionLeft'] ?? -1;
+		$request['selectionBottom']= $request['selectionBottom'] ?? 10000;
+		$request['selectionRight']= $request['selectionRight'] ?? 10000;
 		$selectionTop=max( (int)($request['selectionTop']), 1);
 		$selectionLeft=max( (int)($request['selectionLeft']), 1);
 		$selectionBottom=min( (int)($request['selectionBottom']), $rows);
@@ -222,12 +234,12 @@
 			}
 			$constructId=$request['constructid'];
 			$construct=construct::getById($constructId);
-			if($plate['projectname']!=project::DEFAULTPROJECT && $construct['projectid'] != $plate['projectid']){
+			if($plate['projectname']!= baseproject::DEFAULTPROJECT && $construct['projectid'] != $plate['projectid']){
 				throw new BadRequestException('You cannot use proteins from projects other than the plate project');
 			}
 			$wasAdmin=session::isAdmin();
 			session::set('isAdmin', true);
-			if($plate['projectname']==project::DEFAULTPROJECT){
+			if($plate['projectname']== baseproject::DEFAULTPROJECT){
 				static::setProjectId($id, $construct['projectid']);
 			}
 			$params=array(':constructid'=>$constructId, 'projectid'=>$construct['projectid']);
@@ -277,7 +289,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function delete($id){
+	public static function delete($id): array {
 		if(!self::canDelete($id)){
 			throw new ForbiddenException('Either this plate does not exist or you do not have permission to delete it');
 		}
@@ -297,7 +309,8 @@
      * @noinspection PhpUnusedParameterInspection
      */
 	public static function getconstructs($id,$request=array()){
-		static::getById($id); //throws a NotFoundException if can't read, so we don't need to complicate the nex query with a project clause
+		$plate=static::getById($id);
+		if(!$plate){ throw new NotFoundException('Plate not found'); }
 		$sqlStatement='SELECT welldrop.constructid 
 				FROM platewell, welldrop, construct 
 				WHERE welldrop.platewellid=platewell.id 
@@ -327,8 +340,7 @@
      * @noinspection PhpUnused
      */
 	public static function getplatewells($id,$request=array()){
-		$request['pagenumber']=1;
-		$request['pagesize']=100000;
+		$request['all']=1;
 		$request['sortby']='name';
 		return platewell::getByProperty('plateid', $id, $request);
 	}
@@ -342,13 +354,13 @@
      * @throws ServerException
      */
 	public static function getwelldrops($id,$request=array()){
-		$request['pagenumber']=1;
-		$request['pagesize']=100000;
+		$request['all']=1;
 		$request['sortby']='name';
 		$sqlStatement='SELECT platewell.row AS `row`, platewell.col AS `col`, welldrop.id AS id,
  				welldrop.dropnumber as dropnumber, welldrop.constructid AS constructid, 
  				welldrop.proteinconcentrationamount, welldrop.proteinconcentrationunit, welldrop.proteinbuffer,
 				welldrop.proteinsolutionamount, welldrop.proteinsolutionunit, 
+				welldrop.latestcrystalscoreid, welldrop.latestcrystalprobabilitypercent,
 				welldrop.wellsolutionamount, welldrop.wellsolutionunit, project.id AS projectid
 				FROM platewell, welldrop, project
 				WHERE platewell.plateid=:plateid
@@ -390,7 +402,6 @@
      */
 	public static function getcrystals($id,$request=array()){
 		return crystal::getByProperty('plateid', $id, $request);
-
 	}
 
     /**
@@ -413,6 +424,7 @@
      * @param $projectId
      * @throws BadRequestException
      * @throws ServerException
+     * @throws NotFoundException
      */
 	private static function setProjectId($plateId,$projectId){
 		$result1=database::queryGetAll('SELECT plate.id as pid, platewell.id AS wid, welldrop.id AS did 
@@ -454,8 +466,14 @@
 				WHERE plate.id=:plateid', 
 				array(':plateid'=>$plateId, 'projectid'=>$projectId) 
 		);
+		plate::updateFileNoteAndRecordingProjectIds($plateId);
 	}
 
+    public static function updateFileNoteAndRecordingProjectIds($parentId){
+	    parent::updateFileNoteAndRecordingProjectIds($parentId);
+    }
+
+
     /**
      * Whether the current user can delete the specified plate.
      * @param int $id The ID of the plate.
@@ -464,7 +482,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function canDelete($id){
+	public static function canDelete($id): bool {
 		if(parent::canDelete($id)){ return true; }
 		$plate=plate::getById($id);
 		if($plate['ownerid']==session::getUserId()){
@@ -484,8 +502,6 @@
      * @noinspection PhpUnused
      */
 	public static function getdropbestscores($id,$request=array()){
-		$request['pagenum']=1;
-		$request['pagesize']=10000;
 		$plate=static::getById($id);
 		$scoringSystemId=$plate['crystalscoringsystemid'];
 		$scores=crystalscoringsystem::getscores($scoringSystemId);
@@ -608,7 +624,7 @@
      */
 	public static function destroyByName($name, $request=array()){
         $dateDestroyed=gmdate('Y-m-d');
-	    if(isset($request) && isset($request['datedestroyed'])){
+	    if(isset($request['datedestroyed'])){
 	        $dateDestroyed=$request['datedestroyed'];
 	    }
 	    $wasAdmin=session::isAdmin();
@@ -620,5 +636,60 @@
 	    }
 	    return plate::updateDestroyed($plate['id'], $dateDestroyed);
 	}
-	
+
+    /**
+     * @param $plateId
+     * @return void
+     * @throws BadRequestException
+     * @throws ForbiddenException
+     * @throws NotFoundException
+     * @throws NotModifiedException
+     * @throws ServerException
+     */
+    public static function calculateBestScores($plateId){
+        $plateBestScoreId=database::$nullValue;
+        $plateBestProbabilityPercent=database::$nullValue;
+
+        $wasAdmin=session::isAdmin();
+        if(!$wasAdmin){ session::becomeAdmin(); } //plate might be in default project
+
+        $plate=static::getById($plateId);
+        if(!$plate){ throw new NotFoundException('No plate found with ID '.$plateId); }
+
+        //Get the possible scores for the plate's scoring system, and sort best-first
+        $scores=crystalscoringsystem::getscores($plate['crystalscoringsystemid']);
+        if(!$scores){ return; }
+        $scores=$scores['rows'];
+        usort($scores, function ($a, $b){
+            if($a==$b) { return 0; }
+            return (1*$a['scoreindex']>1*$b['scoreindex']) ? -1 : 1;
+        });
+        //Get the distinct scores for all the drops in the plate
+        $drops=plate::getwelldrops($plate['id'])['rows'];
+        $dropScores=array_unique(array_column($drops, 'latestcrystalscoreid'));
+
+        //Best score first, see if each score exists on the plate. If yes, this is the plate best score, so stop.
+        foreach($scores as $score){
+            if(in_array($score['id'], $dropScores)){
+                $plateBestScoreId=$score['id'];
+                break;
+            }
+        }
+
+        //Get the crystal probabilities, sort descending, choose the first. If none/empty, set null.
+        $dropProbabilities=array_column($drops,'latestcrystalprobabilitypercent');
+        if(!empty($dropProbabilities)){
+            rsort($dropProbabilities, SORT_NUMERIC);
+            $plateBestProbabilityPercent=$dropProbabilities[0];
+        }
+        if(!$plateBestProbabilityPercent){ $plateBestProbabilityPercent=database::$nullValue; }
+
+        //update the plate's best score and highest probability
+        plate::update($plateId, array(
+            'bestprobability'=>$plateBestProbabilityPercent,
+            'bestcrystalscoreid'=>$plateBestScoreId
+        ));
+        if(!$wasAdmin){ session::revokeAdmin(); }
+    }
+
 }
\ No newline at end of file
