@@ -25,7 +25,7 @@
      * @throws BadRequestException
      * @throws NotFoundException
      */
-	public static function canCreate(){
+	public static function canCreate(): bool {
 		if(session::isAdmin()){ return true; }
 		$grp=usergroup::getByName(usergroup::TECHNICIANS);
 		return usergroup::userisingroup($grp['id']);
