@@ -5,6 +5,7 @@
 			'manufacturer'=>validator::ANY,
 			'catalognumber'=>validator::ANY,
 			'isstandard'=>validator::BOOLEAN,
+            'isavailable'=>validator::BOOLEAN
 	);
 	
 	protected static $helpTexts=array(
@@ -12,13 +13,14 @@
 			'manufacturer'=>'The manufacturer of the screen',
 			'catalognumber'=>'Manufacturer\'s catalogue number',
 			'isstandard'=>'Whether this is a standard screen (not an optimization)',
+            'isavailable'=>'Whether this screen is available for use (affects standard screens only)'
 	);
 
 	public static $defaultSortOrder='screen.isstandard DESC, name ASC';
 	
-	private static $rowLabels=array('-','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');
+	private static array $rowLabels=array('-','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');
 	
-	private static $csvMimetypes = array(
+	private static array $csvMimetypes = array(
 	    'text/csv',
 	    'text/plain',
 	    'application/csv',
@@ -38,10 +40,13 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function canCreateStandardScreens(){
+	public static function canCreateStandardScreens(): bool {
 		$projects=session::getCreateProjects();
 		if(empty($projects)){ return false; }
-		$sharedProject=project::getByName(project::SHARED);
+		$sharedProject=project::getByName(baseproject::SHARED);
+		if(!$sharedProject){
+			throw new NotFoundException('Could not read shared project');
+		}
 		return in_array($sharedProject['id'], $projects);
 	}
 
@@ -53,110 +58,155 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function canUpdate($id){
+	public static function canUpdate($id): bool {
 		 if(parent::canUpdate($id)){ return true; }
 		 $groupId=usergroup::getByName(usergroup::TECHNICIANS);
 		 return usergroup::userisingroup($groupId);
 	}
 
+    /**
+     * @param array $request
+     * @return array
+     * @throws BadRequestException
+     * @throws ForbiddenException
+     * @throws NotFoundException
+     * @throws NotModifiedException
+     * @throws ServerException
+     */
+	public static function create(array $request=array()): array {
 
-	public static function create($request=array()){
-		if(isset($request['projectid'])){
-			project::getById($request['projectid']); //auth check
-			$request['isstandard']=false;
-		} else {
-			$project=project::getByName(project::SHARED);
-			$request['projectid']=$project['id'];
-			$request['isstandard']=true;
-		}
-		if(isset($request['conditions'])){
-			//Installer passes in a 12x8 array of standard screen conditions with the screen
-			$conditions=$request['conditions'];
-		} else if(empty($_FILES)){
-			//No file uploaded and no conditions passed in
-			$rows=$request['rows'];
-			$cols=$request['cols'];
-			$conditions=static::makeDummyScreenConditions($rows, $cols, 'Unknown');
-		} else {
-			//File was uploaded, need to parse its conditions
-			if(!isset($request['name'])){
-				//Screen name is file name if none provided
-				$filename=$_FILES['file']['name'];
-                $request['name']=pathinfo($filename, PATHINFO_FILENAME);
+        //Is this a standard screen or an optimization screen?
+        if (isset($request['projectid'])) {
+            $project=project::getById($request['projectid']);
+			if(!$project){
+				throw new NotFoundException('Project does not exist, or you do not have permission to see it');
 			}
-			$fileContent=file_get_contents($_FILES['file']['tmp_name']);
-			if(false!==strpos(strtolower($fileContent),'rock maker xml')){
-				//Formulatrix Rock Maker XML file
-				$conditions=static::parseRockMakerXml($fileContent);
-			    $request['manufacturer']=static::$rockMakerXmlScreenManufacturerName;
-			} else if(false!==strpos($fileContent, 'CrysScreen')){
-				//Tecan CrysScreen Word document (ugh)
-				$conditions=static::parseTecanCrysScreenWordFile($fileContent);
-			} else if(false!==strpos($fileContent, 'crystaltrak ')){
-				//Rigaku CrystalTrak XML file
-				$conditions=static::parseCrystalTrakXml($fileContent);
-			} else if(in_array($_FILES['file']['type'], static::$csvMimetypes)){
-				//Hopefully a CSV file. But if so, what kind?
-				
-				$delimiter=static::determineCsvDelimiter($fileContent);
-				$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
-				$firstRow=str_getcsv($fileContent[0], $delimiter);
-				$thirdRow=str_getcsv($fileContent[2], $delimiter);
-				
-				if(2==count($thirdRow)){
-					//Simple 2-column "well number, condition" CSV 
-					$conditions=static::parseWellNameAndConditionCsv($fileContent);
-				} else if(2<count($thirdRow)){
-					//Hopefully a Mimer CSV...
-					$conditions=static::parseMimerCsv($fileContent);
-					if(!isset($request['name'])){
-						if(empty($firstRow[0])){
-							throw new BadRequestException('No name supplied, and could not find one in cell A1');
-						}
-						$request['name']=$firstRow[0];
-					}
-				} else {
-					//Only one column, if it's a CSV it doesn't look like it
-					throw new BadRequestException('File does not look like a screen - not enough columns');
-				}
-				
-			} else {
-				//Definitely not a recognised format, can't parse conditions - just attach it and "see file"
-				$rows=$request['rows'];
-				$cols=$request['cols'];
-				$conditions=static::makeDummyScreenConditions($rows, $cols, 'See screen definition file');
+			$request['isstandard'] = false;
+            if(isset($request['plateid'])){
+                $plate=plate::getById($request['plateid']);
+                if(!$plate){
+                    throw new BadRequestException('No plate with ID '.$request['plateid']);
+                }
+                $barcode=$plate['name'];
+                $screenName='Optimization '.$barcode;
+                $existing=screen::getByName($screenName);
+                if($existing){
+                    screen::delete($existing['id']);
+                }
+                $request['name']=$screenName;
+            }
+        } else {
+            $project = project::getByName(baseproject::SHARED);
+			if(!$project){
+				throw new NotFoundException('Could not read shared project');
 			}
-		}
+            $request['projectid'] = $project['id'];
+            $request['isstandard'] = true;
+        }
+        if (isset($_FILES['file'])) {
+            //A file was uploaded
+            $filename = $_FILES['file']['name'];
+            $tempName = $_FILES['file']['tmp_name'];
+            $request['tempfile'] = rtrim(config::get('core_filestore'), '/') . '/' . $filename;
+            if (!@move_uploaded_file($tempName, $request['tempfile'])) {
+                throw new ServerException('Could not move file into temporary storage');
+            }
+        }
+        if (!isset($request['tempfile'])) {
+            //There is no file with dimensions and conditions
+            $parsedFile=static::makeDummyScreenConditions($request['rows'], $request['cols'], 'Unknown');
+        } else {
+            //Either a file was uploaded ($_FILES['file'] above) or a file was already on disk, for example from
+            //the Formulatrix screen library in screenfromrockmaker::create.
+            $parsedFile=static::parseScreenFile($request['tempfile'], $request);
+        }
 
-		//Create the screen...
-		$screen=parent::createByClassName($request,'screen');
-		$screenId=$screen['created']['id'];
-		
-		//...and its conditions...
-		static::createConditionsFromPlateArray($conditions, $screenId);
+        foreach(array('name','rows','cols') as $key){
+            if(isset($parsedFile[$key]) && !isset($request[$key])){
+                $request[$key]=$parsedFile[$key];
+            }
+        }
+
+        //Create the screen...
+        $screen=parent::createByClassName($request,'screen');
+        $screenId=$screen['created']['id'];
+
+        //...and its conditions...
+        static::createConditionsFromPlateArray($parsedFile['conditions'], $screenId);
+
+        if(isset($request['plateid'])){
+            plate::update($request['plateid'], array('screenid'=>$screenId));
+        }
+
+        //Create DB record of file and save it
+        if(!empty($request['tempfile'])){
+            $tempFile=$request['tempfile'];
+            file::createFromTempFile(array(
+                'description'=>'Screen definition file',
+                'parentid'=>$screenId,
+                'filename'=>pathinfo($tempFile, PATHINFO_FILENAME).'.'.pathinfo($tempFile, PATHINFO_EXTENSION),
+                'mimetype'=>mime_content_type($tempFile),
+                'tempfile'=>$tempFile
+            ));
+        }
+        return $screen;
+    }
+
+    /**
+     * @param $filePath
+     * @param $request
+     * @return array
+     * @throws BadRequestException
+     * @throws ServerException
+     */
+    private static function parseScreenFile($filePath, $request): array {
+
+        $fileContent=@file_get_contents($filePath);
+        if(!$fileContent){
+            throw new ServerException('Could not open temporary file at '.$filePath.' to parse conditions');
+        }
+        if(str_contains(strtolower($fileContent), 'rock maker xml')){
+            //Formulatrix Rock Maker XML file
+            $ret=static::parseRockMakerXml($fileContent);
+        } else if(str_contains($fileContent, 'CrysScreen')){
+            //Tecan CrysScreen Word document (ugh)
+            $ret=static::parseTecanCrysScreenWordFile($fileContent);
+        } else if(str_contains($fileContent, 'crystaltrak ')){
+            //Rigaku CrystalTrak XML file
+            $ret=static::parseCrystalTrakXml($fileContent);
+        } else if(in_array($_FILES['file']['type'], static::$csvMimetypes)) {
+            //Hopefully a CSV file. But if so, what kind? Could be Mimer, or simple 2-column condition list
+            $ret = static::parseCsvFile($fileContent);
+        } else {
+            $rows=1*$request['rows'];
+            $cols=1*$request['cols'];
+            if(1>$rows || 1>$cols){
+                throw new ServerException('Cannot parse uploaded file. Cannot create dummy conditions - bad rows/columns numbers');
+            }
+            $ret=static::makeDummyScreenConditions($rows, $cols, 'See screen definition file');
+        }
+        if(!isset($ret['name'])){ $ret['name']=pathinfo($filePath, PATHINFO_FILENAME); }
+        if(!isset($ret['rows'])){ $ret['rows']=$request['rows']; }
+        if(!isset($ret['cols'])){ $ret['cols']=$request['cols']; }
+	    return $ret;
+    }
 
-		//Create DB record of file and save it
-		if(!empty($_FILES) && !empty($_FILES['file'])){
-		    file::create(array(
-				'description'=>'Screen definition file',
-				'parentid'=>$screenId,
-    		));
-		}
-		return $screen;
-	}
 
     /**
      * @param $id
      * @param array $request
-     * @return array|void
-     * @throws BadRequestException
+     * @return array
+	 * @throws BadRequestException
      * @throws ForbiddenException
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function update($id, $request=array()){
+	public static function update($id, $request=array()): array {
 		if(isset($request['isstandard']) && $request['isstandard']){
-			$sharedProject=project::getByName(project::SHARED);
+			$sharedProject=project::getByName(baseproject::SHARED);
+			if(!$sharedProject){
+				throw new NotFoundException('Could not read shared project');
+			}
 			$parameters=array(':projectid'=>$sharedProject['id'], ':id'=>$id);
 			database::query(
 					'UPDATE baseobject set projectid=:projectid WHERE id=:id', 
@@ -176,14 +226,13 @@
 
     /**
      * @param $id
-     * @param $request
+     * @param array $request
      * @return array
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      * @noinspection PhpUnused
      */
-	public static function getscreenconditions($id,$request=array()){
+	public static function getscreenconditions($id, array $request=array()): array {
 		$ret=screencondition::getByProperty('screenid', $id, $request);
 		$screen=static::getById($id);
 		$ret['screen']=$screen;
@@ -192,13 +241,12 @@
 
     /**
      * @param $id
-     * @param $request
-     * @return array
+     * @param array $request
+     * @return array|null
      * @throws BadRequestException
-     * @throws NotFoundException
-     * @throws ServerException
+	 * @throws ServerException
      */
-    public static function getplates($id, $request=array()){
+    public static function getplates($id, array $request=array()): ?array {
 	    return plate::getByProperty('screenid', $id, $request);
     }
 
@@ -211,7 +259,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-    public static function delete($id){
+    public static function delete($id): array {
         $screen=static::getById($id);
         if(!$screen){
             throw new BadRequestException('No screen with ID '.$id.', cannot delete');
@@ -245,8 +293,12 @@
         $files=static::getFiles($id);
         if($files){
             foreach($files['rows'] as $file){
-                file::delete($file['id']);
-                $deleted[]=$file['id'];
+                try {
+                    file::delete($file['id']);
+                    $deleted[]=$file['id'];
+                } catch (NotFoundException){
+                    //do nothing, already gone
+                }
             }
         }
         baseobject::delete($id);
@@ -259,8 +311,38 @@
      * @return array
      * @throws BadRequestException
      */
-	private static function parseCrystalTrakXml($fileContent){
-		global $request; //ugly
+    private static function parseCsvFile($fileContent): array {
+        $delimiter=static::determineCsvDelimiter($fileContent);
+        $fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
+        $firstRow=str_getcsv($fileContent[0], $delimiter);
+        $thirdRow=str_getcsv($fileContent[2], $delimiter);
+
+        if(2==count($thirdRow)){
+            //Simple 2-column "well number, condition" CSV
+            $ret=static::parseWellNameAndConditionCsv($fileContent);
+        } else if(2<count($thirdRow)){
+            //Hopefully a Mimer CSV...
+            $ret=static::parseMimerCsv($fileContent);
+            if(!isset($ret['name'])){
+                if(empty($firstRow[0])){
+                    throw new BadRequestException('No name supplied, and could not find one in cell A1');
+                }
+                $ret['name']=$firstRow[0];
+            }
+        } else {
+            //Only one column, if it's a CSV it doesn't look like it
+            throw new BadRequestException('File does not look like a screen - not enough columns');
+        }
+
+        return $ret;
+    }
+
+    /**
+     * @param $fileContent
+     * @return array
+     * @throws BadRequestException
+     */
+	private static function parseCrystalTrakXml($fileContent): array {
 		$xml=simplexml_load_string($fileContent);
 		if(!$xml){ throw new BadRequestException('Detected CrystalTrak XML file but it was not valid XML'); }
 		if(!empty($xml->reservoir_design) && !empty($xml->reservoir_design->format->attributes())){
@@ -277,12 +359,9 @@
 		if(empty($format['rows']) || empty($format['cols'])){
 			throw new BadRequestException('Rows and columns not defined in screen file');
 		}
-		$rows=$format['rows'];
-		$cols=$format['cols'];
-		$request['name']=$screenName;
-		$request['rows']=$rows;
-		$request['cols']=$cols;
-		
+		$rows=(int)$format['rows'];
+		$cols=(int)$format['cols'];
+
 		$conditions=array();
 		for($r=1;$r<=$rows;$r++){
 			$conditions[]=array();
@@ -313,12 +392,14 @@
 			$conditions[$rowNumber-1][$colNumber-1]=$description;
 
 		}
-		return $conditions;
+		return array(
+		    'name'=>$screenName,
+		    'rows'=>$rows,
+		    'cols'=>$cols,
+		    'conditions'=>$conditions
+        );
 	}
 
-
-	private static $rockMakerXmlScreenManufacturerName='';
-
     /**
      * Parses a RockMaker XML screen file.
      * Schema is at http://help.formulatrix.com/rock-maker/3.10/Content/RMXML_Schema.htm as of March 2020; it has been
@@ -336,7 +417,7 @@
      * @throws BadRequestException if the file does not look like an RMXML file, or if it does not contain exactly 96 conditions
      * @throws ServerException if the simpleXML PHP extension is not available
      */
-	private static function parseRockMakerXml($fileContent){
+	private static function parseRockMakerXml(string $fileContent): array {
 	    if(!function_exists('simplexml_load_string')){
 	        throw new ServerException('Cannot process XML file because simplexml is not available. Your administrator needs to fix this.');
         }
@@ -348,6 +429,7 @@
 
         //First pull all the ingredients into an array, identified by their LocalID.
         //We will need to guess the screen manufacturer from the ingredient manufacturer!
+        $screenManufacturer='';
         $ingredients=array();
         foreach($xml->ingredients->children() as $i){
             $stocks=$i->stocks->children();
@@ -370,10 +452,10 @@
                 //At least one screen (from Kerafast, see screens link in function header) contains stock from two or
                 //more manufacturers.
                 $stockManufacturer=$stock->vendorName.'';
-                if(''==static::$rockMakerXmlScreenManufacturerName){
-                    static::$rockMakerXmlScreenManufacturerName=$stockManufacturer;
-                } else if(static::$rockMakerXmlScreenManufacturerName!=$stockManufacturer){
-                    static::$rockMakerXmlScreenManufacturerName='';
+                if(''==$screenManufacturer){
+                    $screenManufacturer=$stockManufacturer;
+                } else if($screenManufacturer!=$stockManufacturer){
+                    $screenManufacturer='';
                 }
             }
         }
@@ -431,7 +513,12 @@
             $conditions[$row][$col]=implode('; ',$parts);
             $count++;
         }
-        return $conditions;
+        return array(
+            'rows'=>$rows,
+            'cols'=>$cols,
+            'manufacturer'=>$screenManufacturer,
+            'conditions'=>$conditions
+        );
 	}
 
     /**
@@ -439,7 +526,7 @@
      * @return array
      * @throws BadRequestException
      */
-	private static function parseMimerCsv($fileContent){
+	private static function parseMimerCsv($fileContent): array {
 		/* Example CSV format: note that the column order may be different and row 1 may not have the numbers.
 		Incomplete factorial,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
 		Reagent number,PEG 8000,[PEG 8000],[PEG 8000] units,pH_PEG 8000,calcium acetate,[calcium acetate],[calcium acetate] units,pH_calcium acetate,calcium acetate,[calcium acetate],[calcium acetate] units,pH_calcium acetate,"bistris propane, pH7","[bistris propane, pH7]","[bistris propane, pH7] units","pH_bistris propane, pH7","cacodylate, pH 6.5","[cacodylate, pH 6.5]","[cacodylate, pH 6.5] units","pH_cacodylate, pH 6.5",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
@@ -470,7 +557,7 @@
 			if(!isset($secondRow[$i])){ break; }
 			$label=trim($secondRow[$i]);
 			if(empty($label)){ break; }
-			if(false!==strpos($label,'[') || 'pH'==$label || 0===strpos($label,'pH_')){ continue; }
+			if(str_contains($label, '[') || 'pH'==$label || str_starts_with($label, 'pH_')){ continue; }
 			$parts[$label]=array('name'=>$i);
 		}
 		//1b. Now iterate through the headers again, trying to find the column numbers for each component's pH, amount, units 
@@ -478,13 +565,13 @@
 			if(!isset($secondRow[$i])){ break; }
 			$label=trim($secondRow[$i]);
 			if(empty($label)){ break; }
-			if(false!==strpos($label,'[')){
-				if('] units'==substr($label,-7)){
+			if(str_contains($label, '[')){
+				if(str_ends_with($label, '] units')){
 					$parts[substr($label,1,-7)]['unit']=$i;
-				} else if(']'==substr($label,-1)){
+				} else if(str_ends_with($label, ']')){
 					$parts[substr($label,1,-1)]['amount']=$i;
 				} 
-			} else if(false!==strpos($label,'pH_')){
+			} else if(str_contains($label, 'pH_')){
 				$parts[substr($label,3)]['pH']=$i;
 			} else if('pH'==$label){
 				//This is a pain. The label says nothing about what this is the pH *of*.
@@ -492,7 +579,7 @@
 				for($j=0;$j<$i;$j++){
 					$otherCol=$i-$j;
 					$otherLabel=trim($secondRow[$otherCol]);
-					if(']'==substr($otherLabel,-1)){
+					if(str_ends_with($otherLabel, ']')){
 						$parts[substr($otherLabel,1,-1)]['pH']=$i;
 						break;
 					}
@@ -520,12 +607,15 @@
 			//Text description
 			$labelParts=array();
 			foreach($parts as $k=>$v){
+				if(!isset($v['amount'])){
+					throw new BadRequestException('Badly-formed CSV. Does not look like simple 2-column file. Failed to parse as a MIMER file.');
+				}
 				if(!isset($row[$v['amount']]) || !isset($row[$v['unit']]) || !isset($row[$v['name']])){
 					continue;
 				}
 				$str=trim($row[$v['amount']].$row[$v['unit']].' '.$row[$v['name']]);
 				if(!empty($str)){
-					if(isset($v['pH']) && !empty($row[$v['pH']]) && false==strpos($str,'pH')){
+					if(isset($v['pH']) && !empty($row[$v['pH']]) && !str_contains($str, 'pH')){
 						$str.=', pH '.$row[$v['pH']];
 					}
 					$labelParts[]=$str;
@@ -533,9 +623,9 @@
 			}
 			$label=implode('; ',$labelParts);
 			$well=array(
-					'description'=>$label,
-					'row'=>$rowNumber,
-					'col'=>$columnNumber
+				'description'=>$label,
+				'row'=>$rowNumber,
+				'col'=>$columnNumber
 			);
 			$wells[]=$well;
 		}
@@ -554,18 +644,22 @@
 			$conditions[$r-1][$c-1]=$w['description'];
 		}
 		
-		return $conditions;
+		return array(
+		    'rows'=>$rows,
+		    'cols'=>$cols,
+		    'conditions'=>$conditions
+        );
 	}
 
 	/**
 	 * Parses a Tecan CrysScreen Word document's well conditions.
 	 * Assumes that the document has a table with B:(buffer) P:(precipitant) S:(salt) in each cell, and that the wells appear
 	 * in the document in A1-A12, B1-B12, etc., order. 
-	 * @param string $fileContent The contents of the file
+	 * @param string|array $fileContent The contents of the file
 	 * @return array A 2D array of screen conditions
 	 * @throws BadRequestException if file does not contain exactly 96 conditions
 	 */
-	private static function parseTecanCrysScreenWordFile($fileContent){
+	private static function parseTecanCrysScreenWordFile(string|array $fileContent): array {
 	
 		if(!is_array($fileContent)){
 			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
@@ -595,7 +689,6 @@
         $wholeScreen=str_replace('[[[SEP]]][[[SEP]]]', '[[[SEP]]]', $wholeScreen); //Table rows delimited by double separator, change to single
 		$parts=explode('[[[SEP]]]', $wholeScreen);
 		if(count($parts)!=96){
-		    echo $wholeScreen;
 			throw new BadRequestException('Could not parse CrysScreen file. Expected 96 conditions, but found '.count($parts).'.');
 		}
 
@@ -613,21 +706,24 @@
 			$conditions[$row][$col]=$condition;
 		}
 		
-		return $conditions;
+		return array(
+		    'rows'=>8,
+		    'cols'=>12,
+		    'conditions'=>$conditions
+        );
 	}
 
-    /**
-     * Parses a CSV file with just well and condition into the database
-     * A1,Condition
-     * A01,Condition
-     * 1,Condition - assumes numbering A1-A12,B1-B12, etc.
-     * @param string $fileContent
-     * @return array
-     * @throws BadRequestException if file does not contain exactly 96 conditions or any well ID is outside 12x8
-     */
-	public static function parseWellNameAndConditionCsv($fileContent){
-		//Public because installer uses it.
-	
+	/**
+	 * Parses a CSV file with just well and condition
+	 * A1,Condition
+	 * A01,Condition
+	 * 1,Condition - assumes numbering A1-A12,B1-B12, etc.
+	 * @param string|array $fileContent
+	 * @return array
+	 * @throws BadRequestException if file does not contain exactly 96 conditions or any well ID is outside 12x8
+	 */
+	private static function parseWellNameAndConditionCsv(string|array $fileContent): array {
+
 		if(!is_array($fileContent)){
 			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
 		}
@@ -641,51 +737,29 @@
 		//First pass: Determine row and column dimensions of screen (usually 8x12)
 		$maxRowNumber=0;
 		$maxColNumber=0;
-		$wellCount=0;
 		foreach($fileContent as $row){
 		    $row=str_getcsv($row, $delimiter);
 		    if(empty($row) || empty($row[0])){ continue; } //blank line at end of file
-            if(!preg_match('/^[A-Z]?\d\d?$/',$row[0])){ continue; } //header row
+			$wellLabel=trim($row[0]);
+            if(!preg_match('/^[A-Z]\d\d?$/',$wellLabel)){ continue; } //header row
 
             //Row and column
-		    $wellLabel=trim($row[0]);
-		    if(preg_match('/^[A-Z]?\d\d?$/',$wellLabel)){
-		        $wellCount++;
-		    } else {
-		        $wellNumber=(int)$wellLabel;
-		        $wellCount=max($wellCount,$wellNumber);
-		    }
+			$rowNumber=array_search(substr($wellLabel,0,1), static::$rowLabels);
+			$colNumber=(int)(substr($wellLabel,1));
+			$maxRowNumber=max($maxRowNumber, $rowNumber);
+			$maxColNumber=max($maxColNumber, $colNumber);
 		}
 
-		//If column 1 contains well numbers (1-96) not positions (A[0]1-H12), we have to make some
-		//assumptions about the plate format. If these are incorrect, then we will barf when applying the
-		//screen to the plate. Hey, we tried.
-		if(0==$maxColNumber || 0==$maxRowNumber){
-		    if(96==$wellCount){
-		        $maxRowNumber=8;
-		        $maxColNumber=12;
-		    } else if(48==$wellCount){
-		        $maxRowNumber=6;
-		        $maxColNumber=8;
-		    } else if(24==$wellCount){
-		        $maxRowNumber=4;
-		        $maxColNumber=6;
-		    } else if(12==$wellCount){
-		        $maxRowNumber=4;
-		        $maxColNumber=3;
-		    } else {
-		        throw new BadRequestException('Could not determine screen XY dimensions from CSV. Found '.$wellCount.' conditions. Use A01, A02, etc., in column 1');
-		    }
-		}
-		
+
 		//Make a 2d array matching the screen dimensions, all cells containing ''
-		$conditions=array_fill(0,$maxRowNumber,array_fill(0,$maxColNumber,''));
+		$conditions=array_fill(0,$maxRowNumber,array_fill(0,$maxColNumber,'-'));
 		
 		//Second pass: Fill the 2d array with conditions
+        $rows=0;
+        $cols=0;
 		foreach($fileContent as $row){
 			$row=str_getcsv($row, $delimiter);
 			if(empty($row) || empty($row[0])){ continue; } //blank line at end of file
-			//if(strtolower('Well')==strtolower($row[0])){ continue; } //header row
             if(!preg_match('/^[A-Z]?\d\d?$/',$row[0])){ continue; } //header row
 
             //Row and column
@@ -705,8 +779,14 @@
 				throw new BadRequestException('Well identifier ('.$wellLabel.') not recognised or out of bounds');
 			}
 			$conditions[$rowNumber-1][$colNumber-1]=$condition;
+			$rows=max($rows,$rowNumber);
+			$cols=max($cols,$colNumber);
 		}
-		return $conditions;
+		return array(
+		    'rows'=>$rows,
+		    'cols'=>$cols,
+		    'conditions'=>$conditions
+        );
 	}
 
     /**
@@ -714,20 +794,30 @@
      * @param $cols
      * @param $dummyText
      * @return array
+     * @throws BadRequestException
      */
-	private static function makeDummyScreenConditions($rows, $cols, $dummyText){
+	private static function makeDummyScreenConditions($rows, $cols, $dummyText): array {
+	    $rows=(int)$rows;
+	    $cols=(int)$cols;
+	    if(1>$rows || 1>$cols){
+	        throw new BadRequestException('Bad row/column count in makeDummyScreenConditions');
+        }
 		$conditions=array();
 		$separator='|---|';
 		$rowText=str_repeat($dummyText.$separator, $cols-1) . $dummyText;
 		for($r=1;$r<=$rows;$r++){
 			$conditions[]=explode($separator, $rowText);
 		}
-		return $conditions;
+        return array(
+            'rows'=>$rows,
+            'cols'=>$cols,
+            'conditions'=>$conditions
+        );
 	}
 
 
     /**
-     * Takes an (8x12) array of screen conditions and a screen ID, and sets the screen conditions against the screen.
+     * Takes an 8x12 array of screen conditions and a screen ID, and sets the screen conditions against the screen.
      * @param array $plateArray
      * @param int|string $screenId
      * @throws BadRequestException
@@ -735,11 +825,10 @@
      * @throws ServerException
      * @throws ForbiddenException
      */
-	public static function createConditionsFromPlateArray($plateArray, $screenId){
+	public static function createConditionsFromPlateArray(array $plateArray, int|string $screenId): void {
 		//public because installer uses it
 		$screen=screen::getById($screenId);
 		if(!$screen){ throw new ServerException('screen::createConditionsFromPlateArray got bad screen ID'); }
-		if(!is_array($plateArray)){ throw new ServerException('screen::createConditionsFromPlateArray got non-array'); }
 		$projectId=$screen['projectid'];
 		$rowCount=count($plateArray);
 		$colCount=count($plateArray[0]);
@@ -771,11 +860,11 @@
      * Determines whether a screen CSV file is delimited by comma, semicolon or tab, and returns that delimiter.
      * Note that this implementation is rather naive, assuming that the first column is a well identifier (A1 or A01)
      * or well number (1-96), immediately followed by the delimiter.
-     * @param $fileContent string The content of the file
+     * @param $fileContent string|array The content of the file
      * @return string The delimiter
      * @throws BadRequestException if the delimiter cannot be determined, or is not comma, semicolon, or tab
      */
-	private static function determineCsvDelimiter($fileContent){
+	private static function determineCsvDelimiter(string|array $fileContent): string {
 		if(!is_array($fileContent)){
 			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
 		}
