@@ -28,7 +28,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function create($request=array()){
+	public static function create(array $request=array()): array {
 		$screen=screen::getById($request['screenid']);
 		$projectId=$screen['projectid'];
 		$request['projectid']=$projectId;
@@ -42,7 +42,7 @@
      * @throws ServerException
      * @throws BadRequestException
      */
-	public static function canUpdate($id){
+	public static function canUpdate($id): bool {
 		return screen::canUpdate($id);
 	}
 	
