@@ -30,14 +30,18 @@
     /**
      * Determines whether the current user is in the named usergroup.
      * @param string $groupName The group name.
-     * @return bool true if the user is in the named group, otherwise false.
+     * @return bool true if the named group exists and the user is in it, otherwise false.
      * @throws BadRequestException
-     * @throws NotFoundException if the usergroup does not exist.
+	 * @throws ServerException
      */
-	private static function isA($groupName){
-		$group=usergroup::getByName($groupName);
+	private static function isA(string $groupName): bool {
+		try {
+			$group=usergroup::getByName($groupName);
+		} catch (NotFoundException){
+			return false;
+		}
 		if(!$group){ return false; }
 		return usergroup::userisingroup($group['id']);
 	}
-	
+
 }
\ No newline at end of file
