@@ -1,4 +1,6 @@
-<?php class shipment extends baseobject { 
+<?php use Fpdf\Fpdf;
+
+class shipment extends baseobject {
 	
 	protected static $fields=array(
 		'name'=>validator::REQUIRED,
@@ -40,33 +42,35 @@
                 AND shipment.projectid=project.id
         ';
 	
-	public static function getById($id){
+	public static function getById($id): array {
 	    $shipment=parent::getById($id);
 	    if(!empty($shipment['manifest'])){
 	        $shipment['manifest']=static::parseAndSanitizeManifest($shipment['manifest']);
 	    }
+		$shipment['beamlines']=static::getCollectionBeamlines($id);
 	    return $shipment;
 	}
-	public static function getByName($name){
+	public static function getByName($name): array {
 	    $shipment=parent::getByName($name);
 	    if(!empty($shipment['manifest'])){
 	        $shipment['manifest']=static::parseAndSanitizeManifest($shipment['manifest']);
 	    }
+		$shipment['beamlines']=static::getCollectionBeamlines($shipment['id']);
 	    return $shipment;
 	}
-    public static function getAll($request=array()){
+    public static function getAll(array $request=array()): array {
         $results=parent::getAll($request);
         return static::stripManifestsFromMultipleResults($results);
     }
-    public static function getByNameLike($searchTerms, $request=array()){
+    public static function getByNameLike(string $searchTerms, array $request=array()): array {
         $results=parent::getByNameLike($searchTerms, $request);
         return static::stripManifestsFromMultipleResults($results);
     }
-    public static function getByProperties($keyValuePairs, $request=array()){
+    public static function getByProperties(array $keyValuePairs, array $request=array()): array {
         $results=parent::getByProperties($keyValuePairs, $request);
         return static::stripManifestsFromMultipleResults($results);
     }
-    public static function getByProperty($key, $value, $request=array()){
+    public static function getByProperty($key, $value, $request=array()): array {
         $results=parent::getByProperty($key, $value, $request);
         return static::stripManifestsFromMultipleResults($results);
     }
@@ -83,7 +87,7 @@
      * @throws BadRequestException
      * @throws NotFoundException
      */
-    public static function canCreate(){
+    public static function canCreate(): bool {
 		if(session::isAdmin()){ return true; }
 		$grp=usergroup::getByName(usergroup::TECHNICIANS);
 		if(usergroup::userisingroup($grp['id'])){ return true; }
@@ -98,7 +102,7 @@
      * @throws NotFoundException
      * @throws BadRequestException
      */
-	public static function canUpdate($id){
+	public static function canUpdate($id): bool	{
 	    return static::canCreate();
 	}
 
@@ -110,14 +114,17 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function create($request=array()){
-	    $sharedProject=project::getByName(project::SHARED);
+	public static function create(array $request=array()): array {
+	    $sharedProject=project::getByName(baseproject::SHARED);
+		if(!$sharedProject){
+			throw new NotFoundException('Could not read shared project');
+		}
 	    $sharedProjectId=$sharedProject['id'];
 	    $request['projectid']=$sharedProjectId;
 	    $request['shipperid']=session::getUserId();
 
 	    $containerIds=array();
-	    if(isset($request['dewarbarcodes']) && !empty($request['dewarbarcodes'])){
+	    if(!empty($request['dewarbarcodes'])){
 	        $codes=explode(',',$request['dewarbarcodes']);
 	        foreach($codes as $barcode){
 	            $dewar=container::getByName($barcode);
@@ -125,7 +132,7 @@
 	            $containerIds[]=$dewar['id'];
 	        }
 	    }
-	    if(isset($request['platebarcodes']) && !empty($request['platebarcodes'])){
+	    if(!empty($request['platebarcodes'])){
 	        $codes=explode(',',$request['platebarcodes']);
 	        foreach($codes as $barcode){
 	            $plate=container::getByName($barcode);
@@ -154,6 +161,60 @@
 	    return $shipment;
 	}
 
+	private static function getMxlimsJson($shipment) {
+		include_once __DIR__.'/../../vendor/mxlims/ShipmentEncoder/src/v0_6_7ShipmentEncoder.php';
+		try {
+			$encoder=new mxlims\tools\php\ShipmentEncoder\src\v0_6_7ShipmentEncoder();
+			$proteins=[];
+			$manifest=$shipment['manifest'];
+			if(empty($manifest)){
+				throw new BadRequestException('No manifest, cannot generate MXLIMS JSON');
+			}
+			$sessionNumber=$shipment['sessionname'];
+			//DLS session number might be stored as mx4025-1, we need the 1
+			$sessionNumber=str_replace($shipment['proposalname'].'-', '', $sessionNumber);
+			if(is_numeric($sessionNumber)){
+				//Must be an integer
+				$sessionNumber=(int)$sessionNumber;
+			} else {
+				//might be "" or "Mail-in"
+				$sessionNumber=null;
+			}
+			$encoder->createShipment($shipment['proposalname'], $sessionNumber);
+			session::becomeAdmin(); //need admin to get email
+			$shipper=user::getById($shipment['shipperid']);
+			session::revertAdmin();
+			$encoder->setLabContactOutbound($shipper['fullname'], $shipper['email']);
+			$encoder->setLabContactReturn($shipper['fullname'], $shipper['email']);
+			$dewars=$manifest['rows'];
+			foreach ($dewars as $dewar){
+				$mxlimsDewar=$encoder->addDewarToShipment($dewar['name']);
+				$pucks=$dewar['childitems'];
+				foreach ($pucks as $puck){
+					if(!isset($puck['id'])){ continue; }
+					$mxlimsPuck=$encoder->addPuckToDewar($mxlimsDewar['index'], $puck['name'], $puck['positions']);
+					$pinPosition=-1;
+					foreach ($puck['childitems'] as $pin){
+						$pinPosition++;
+						if(is_string($pin) || !$pin){
+							continue;
+						}
+						$sample=$pin['childitems'][1];
+						$acronym=$sample['proteinacronym'];
+						if(!isset($proteins[$acronym])){
+							$protein=$encoder->addMacromoleculeToShipment($acronym);
+							$proteins[$acronym]=$protein['index'];
+						}
+						$encoder->addSinglePinSampleToPuck($mxlimsPuck['index'], $pinPosition, $proteins[$acronym], $sample['name'], $pin['name']??null);
+					}
+				}
+			}
+			return $encoder->encodeToJSON(true);
+		} catch (Exception $exception){
+			throw new ServerException('MXLIMS shipment encoder threw exception: '.$exception->getMessage());
+		}
+	}
+
     /**
      * @param $id
      * @param array $request
@@ -163,10 +224,13 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	public static function update($id,$request=array()){
+	public static function update($id,$request=array()): array {
 	    $shipment=static::getById($id);
 	    if(!$shipment){ throw new NotFoundException('No shipment with ID '.$id); }
-	    $hasShipped=!empty($shipment['dateshipped']);
+		if(isset($request['getMxlims'])){
+			return ["json"=>static::getMxlimsJson($shipment)];
+		}
+		$hasShipped=!empty($shipment['dateshipped']);
         if(isset($request['shipmentdestinationid'])){
 	        if($hasShipped){ throw new ForbiddenException('Cannot update destination. Shipment has shipped.'); }
 	        $destination=shipmentdestination::getById($request['shipmentdestinationid']);
@@ -180,12 +244,17 @@
        
 	        //update all containercontent where shipment is parent, set shipmentid, recursively through all children
 	        $affected=1;
+	        $passCount=0;
 	        while($affected>0){
 	            $items=database::queryGetAll(
 	                   'SELECT child FROM containercontent WHERE shipmentid=:shipmentid', 
 	                   array(':shipmentid'=>$shipment['id'])
 	            );
-	            if(isset($items['rows'])){
+	            if(!isset($items['rows'])) {
+                    //This can happen if shipment was sent before and shipped date was nulled out for a re-send.
+                    //Likely only in dev/test, but someone might do it in production...
+                    $affected=0;
+                } else {
 	                $children=array_column($items['rows'],'child');
 	                database::query(
 	                     'UPDATE containercontent SET shipmentid=:shipmentid WHERE parent IN('. implode(',',$children) .') AND shipmentid IS NULL',
@@ -194,7 +263,6 @@
 	                $affected=database::getAffectedRows();
 	            }
 	        }
-	        
 	        //prepare shipment::getcontainers() into shipment.manifest as unsanitized JSON - will be sanitized on return to client
 	        $wasAdmin=session::isAdmin();
 	        session::set('isAdmin',true);
@@ -244,7 +312,7 @@
      * @throws Exception
      * @noinspection PhpUnusedParameterInspection
      */
-	public static function getcontainers($id,$request=array()){
+	public static function getcontainers($id, $request=array()) {
 	    $shipment=static::getById($id);
 	    if(!empty($shipment['manifest'])){
 	        return $shipment['manifest'];
@@ -269,19 +337,27 @@
 	    return $ret;
 	}
 
+	private static function getCollectionBeamlines($id) {
+		$beamlines=database::queryGetAll('SELECT DISTINCT bl.id AS id, bl.name AS name
+				FROM diffractionrequest AS dr,dataset AS ds, beamline AS bl 
+				WHERE dr.shipmentid=:shipmentid AND ds.diffractionrequestid=dr.id AND ds.beamlineid=bl.id			
+		',[':shipmentid'=>$id]);
+		if(!$beamlines){ return []; }
+		return $beamlines['rows'];
+	}
+
     /**
      * Takes a JSON encoding of the shipping manifest and unpacks it into an associative array.
      * The manifest will have been generated with administrator permissions and therefore may contain confidential
      * information. If the user is not an administrator, hides information that the user does not have permission to see.
      * @param $manifest
-     * @return mixed
+     * @return array
      */
-	private static function parseAndSanitizeManifest($manifest){
-	    $manifest=json_decode($manifest,true);
-        //TODO Replace anything where projectid is not in read-projects with (hidden)
+	private static function parseAndSanitizeManifest($manifest): array {
+		//TODO Replace anything where projectid is not in read-projects with (hidden)
         //if(!session::isAdmin()){
         //}
-	    return $manifest;
+	    return json_decode($manifest,true);
 	}
 
     /**
@@ -296,7 +372,7 @@
      * @noinspection PhpUnusedParameterInspection
      */
 	public static function getmanifestpdfs($id,$request=array()){
-	    static::generateManifestPdf($id,false);
+	    static::generateManifestPdf($id);
 	}
 
 
@@ -311,7 +387,7 @@
      * @throws NotFoundException
      * @throws ServerException
      */
-	private static function generateManifestPdf($id, $saveAsIceBearFile=false){
+	private static function generateManifestPdf($id, bool $saveAsIceBearFile=false): int|string {
 	    $shipment=static::getById($id);
 	    if(!$shipment){
 	        throw new NotFoundException('No shipment with ID '.$id);
@@ -327,8 +403,7 @@
 	    
 	    $qrWidth=18;
 	    
-	    include_once($_SERVER['DOCUMENT_ROOT'].'/vendor/fpdf/fpdf.php');
-	    $pdf=new FPDF();
+	    $pdf=new Fpdf();
 	    $pdf->SetTitle('IceBear shipment manifest');
 	    
 	    $pdf->SetFillColor(192,192,192);
@@ -342,14 +417,14 @@
         $firstHeaderHeight=20; //mm
         $contentHeight=$pageHeight-$headerHeight-$headerMargin-(2*$margin)-12; //Needs 12mm to fit last pin on each page, no idea why
 	    
-	    $pdf->Image($_SERVER['DOCUMENT_ROOT'].'/client/images/icebearlogo.png',$margin,$margin,0,$firstHeaderHeight);
+	    $pdf->Image(config::getWwwRoot().'/client/images/icebearlogo.png',$margin,$margin,0,$firstHeaderHeight);
 	    $pdf->SetXY($margin+15, $margin);
 	    $pdf->Cell(0,$firstHeaderHeight,'IceBear crystal shipment manifest',1,1,'C',true);
 
 	    $pdf->SetXY($margin, $margin+90);
-	    $pdf->Cell(0,20,'Shipment: '.$shipment['name'],0,1,'C',false);
-	    $pdf->Cell(0,20,'To: '.$shipmentDestination['name'],0,1,'C',false);
-	    $pdf->Cell(0,20,'Shipped: '.$shipment['dateshipped'],0,1,'C',false);
+	    $pdf->Cell(0,20,'Shipment: '.$shipment['name'],0,1,'C');
+	    $pdf->Cell(0,20,'To: '.$shipmentDestination['name'],0,1,'C');
+	    $pdf->Cell(0,20,'Shipped: '.$shipment['dateshipped'],0,1,'C');
 	    
 	    $localUrl=(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']==='on'?'https://':'http://').$_SERVER['HTTP_HOST'].'/shipment/'.$shipment['id'];
 	    $remoteUrl=$shipment['urlatremotefacility'];
@@ -450,7 +525,7 @@
 	                
   	                if(""==$pin){ 
   	                    $pdf->SetXY($pinOriginX+10, $pinOriginY);
-  	                    $pdf->Cell(0,$pinHeight,'(No pin in this position)',1,1,'C',false);
+  	                    $pdf->Cell(0,$pinHeight,'(No pin in this position)',1,1,'C');
   	                    $count++;
   	                    continue; 
   	                } 
@@ -461,14 +536,14 @@
   	                if(stripos($pinBarcode, 'dummypin')===0){
   	                    $pinBarcode='-';
   	                }
- 	                $pdf->Cell(35,7,$pinBarcode,1,1,'C',false);
+ 	                $pdf->Cell(35,7,$pinBarcode,1,1,'C');
  	                //Sample name cell
  	                $pdf->SetXY($pinOriginX+45, $pinOriginY);
- 	                $pdf->Cell(0,7,$crystal['name'],1,1,'C',false);
+ 	                $pdf->Cell(0,7,$crystal['name'],1,1,'C');
  	                //Main cell border
  	                $mainCellWidth=$pageWidth-(2*$margin)-10; //number cell is 10mm wide
  	                $pdf->SetXY($pinOriginX+10, $pinOriginY+7);
- 	                $pdf->Cell(0,$pinHeight-7,'',1,1,'C',false);
+ 	                $pdf->Cell(0,$pinHeight-7,'',1,1,'C');
  	                 	                                
  	                //shipping comment
  	                $pdf->SetXY($pinOriginX+10+$qrWidth, $pinOriginY+12);
@@ -524,10 +599,10 @@
  	                    
      	                $pdf->SetFont('Arial','I',10);
      	                $pdf->SetXY($pinOriginX+10, $pinOriginY+7);
-     	                $pdf->Cell(0,5,'IceBear',0,1,'L',false);
+     	                $pdf->Cell(0,5,'IceBear',0,1,'L');
      	                if($remotePinQr){
          	                $pdf->SetXY($pinOriginX+10, $pinOriginY+7);
-         	                $pdf->Cell(0,5,$shipmentDestination['name'],0,1,'R',false);
+         	                $pdf->Cell(0,5,$shipmentDestination['name'],0,1,'R');
      	                }
      	                $pdf->SetFont('Arial','',12);
      	                @unlink($tempLocal);
@@ -562,5 +637,24 @@
 	    }
 	    
 	}
-	
+
+	/**
+	 * @throws NotFoundException
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function setShipperAdmin(): void {
+		if (!session::isAdmin() && session::isShipper()){
+			session::becomeAdmin();
+		}
+	}
+
+	/**
+	 * @throws BadRequestException
+	 * @throws ServerException
+	 */
+	public static function revertShipperAdmin(): void {
+		session::revertAdmin();
+	}
+
 }
