@@ -1,4 +1,5 @@
-<?php if("done"!=$template){ ?>
+<?php /** @noinspection PhpUndefinedVariableInspection */
+if("done"!=$template){ ?>
 <label> <input type="submit" name="next" id="next" value="Next &gt;" /></label>
 <?php }?>
 </form></body></html>
\ No newline at end of file
