@@ -2,9 +2,9 @@
 $pageTitle="Forbidden";
 include realpath(__DIR__).'/../_common/header.php'; ?>
 <script type="text/javascript">
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
-	var grid=ui.grid();
+	const grid=ui.grid();
 	grid.box({ 
 		classes:'r1 c1 h3 w3',
 		title:'No permission',
