@@ -2,7 +2,7 @@
 include realpath(__DIR__).'/../_common/header.php'; ?>
 <script type="text/javascript">
 
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 	
 	ui.grid();
 
