@@ -1,2 +1,2 @@
 <p>Before going through this installer, you need to run the set-up script. This configures the web server for IceBear, sets up a firewall, and does some basic configuration needed by this installer.</p>
-<p>Run <strong>sudo ./install.sh</strong> in the install directory. Then click Next below.</p>
\ No newline at end of file
+<p>Run <strong>sudo ./install.sh</strong> in the <code>install</code> directory. Then click Next below.</p>
\ No newline at end of file
