@@ -2,18 +2,18 @@
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
 
-document.observe("dom:loaded",function(){
-	var grid=ui.grid();
-	var detailsBox=grid.box({ 
-		classes:'r1 c1 h3 w1',
-		title:'Details'
-	});
+window.addEventListener('DOMContentLoaded',function(){
+    const grid = ui.grid();
+    const detailsBox = grid.box({
+        classes: 'r1 c1 h3 w1',
+        title: 'Details'
+    });
 
-	var frm=detailsBox.form({
-		action:'/api/construct/',
-		method:'post',
-	});
-	frm.textField({ label:"Name", name:'name', value:'' });
+    const frm = detailsBox.form({
+        action: '/api/construct/',
+        method: 'post',
+    });
+    frm.textField({ label:"Name", name:'name', value:'' });
 	frm.textField({ label:"Description", name:'description', value:'' });
 	frm.roleField({ 
 		label:"Protein", 
@@ -28,4 +28,4 @@ document.observe("dom:loaded",function(){
 	
 });
 </script>
-<?php include realpath(__DIR__).'/../../core/_common/footer.php'; ?>
+<?php include realpath(__DIR__).'/../../core/_common/footer.php';
