@@ -1,16 +1,14 @@
-<?php $pageTitle='Construct';
+<?php $pageTitle='Constructs';
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
+    const headers = ['Name', 'Project', 'Description'];
+    const cellTemplates = ['<a href="/construct/{{id}}">{{name}}</a>', [Project.getLink,'name'], '{{description}}'];
 
-	var headers=['Name','Project','Description'];
-	var cellTemplates=['<a href="/construct/{{id}}">{{name}}</a>','<a href="/project/{{projectid}}">{{projectname}}</a>','{{description}}'];
+    const grid = ui.grid();
 
-	var grid=ui.grid();
-
-
-	var infoBox=grid.box({ 
+    grid.box({
 		classes:'r1 c1 h1 w3',
 		content:'This is only for testing and development. Go to the project page for a better way to work with proteins, constructs and sequences.'
 	});
@@ -22,7 +20,6 @@ document.observe("dom:loaded",function(){
 		cellTemplates:cellTemplates
 	});
 
-
 });
 </script>
-<?php include realpath(__DIR__).'/../../core/_common/footer.php'; ?>
\ No newline at end of file
+<?php include realpath(__DIR__).'/../../core/_common/footer.php';
\ No newline at end of file
