@@ -1,11 +1,11 @@
 <?php $pageTitle='Crystal scoring systems';
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
 
-	var headers=['Name','Current'];
-	var cellTemplates=['<a href="/crystalscoringsystem/{{id}}">{{name}}</a>',[ui.checkmark,'iscurrent'] ];
+	let headers=['Name','Current'];
+	let cellTemplates=['<a href="/crystalscoringsystem/{{id}}">{{name}}</a>',[ui.checkmark,'iscurrent'] ];
 	ui.grid().box({ 
 		classes:'r1 c1 h3 w3',
 		url:'/api/crystalscoringsystem',
@@ -16,4 +16,4 @@ document.observe("dom:loaded",function(){
 
 });
 </script>
-<?php include realpath(__DIR__).'/../../core/_common/footer.php'; ?>
\ No newline at end of file
+<?php include realpath(__DIR__).'/../../core/_common/footer.php';
\ No newline at end of file
