@@ -1,20 +1,20 @@
 <?php $pageTitle='Create an imager';
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
-	ui.grid();
+	const grid=ui.grid();
 
-	var detailsBox=grid.box({ 
-		classes:'r1 c1 h3 w1',
-		title:'Details'
-	});
+    const detailsBox = grid.box({
+        classes: 'r1 c1 h3 w1',
+        title: 'Details'
+    });
 
-	var frm=detailsBox.form({
-		action:'/api/imager/',
-		method:'post',
-	});
-	frm.textField({ label:"Serial", name:'name', value:'' });
+    const frm = detailsBox.form({
+        action: '/api/imager/',
+        method: 'post',
+    });
+    frm.textField({ label:"Serial", name:'name', value:'' });
 	frm.textField({ label:"Friendly name", name:'friendlyname', value:'' });
 	frm.textField({ label:"Manufacturer", name:'manufacturer', value:'' });
 	frm.textField({ label:"Temperature", name:'temperature', value:'' });
@@ -23,9 +23,5 @@ document.observe("dom:loaded",function(){
 	frm.textField({ label:"Critical plate loading", name:'warninglevel', value:'' });
 	frm.createButton();
 });
-
-
 </script>
-<?php 
-include realpath(__DIR__).'/../../core/_common/footer.php';
-?>
\ No newline at end of file
+<?php include realpath(__DIR__).'/../../core/_common/footer.php';
\ No newline at end of file
