@@ -1,11 +1,10 @@
 <?php $pageTitle='Create a plate inspection';
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
-	var createEnabled=<?php echo (int)(config::get('imaging_allowmanual')); ?>;
-
-	ui.grid();
+	const createEnabled=<?php echo (int)(config::get('imaging_allowmanual')); ?>;
+	const grid=ui.grid();
 
 	if(!createEnabled){
 		grid.box({ 
