@@ -1,14 +1,13 @@
 <?php $pageTitle='PDB depositions';
+/** @noinspection PhpUnhandledExceptionInspection */
 $showCreateLink=session::isAdmin() || pdbdeposition::canCreate();
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
 	let sortOrders=['name'];
 	let headers=['Name'];
-	let cellTemplates=[
-	               	'<a href="/pdbdeposition/{{id}}">{{name}}</a>',
-	];
+	let cellTemplates=[ '<a href="/pdbdeposition/{{id}}">{{name}}</a>' ];
 
 	ui.grid().box({ 
 		classes:'r1 c1 h3 w3',
@@ -21,4 +20,4 @@ document.observe("dom:loaded",function(){
 
 });
 </script>
-<?php include realpath(__DIR__).'/../../core/_common/footer.php'; ?>
\ No newline at end of file
+<?php include realpath(__DIR__).'/../../core/_common/footer.php';
\ No newline at end of file
