@@ -1,4 +1,4 @@
-document.observe("dom:loaded",function(){
+window.addEventListener('DOMContentLoaded',function(){
 
     ui.grid();
 
