@@ -1,25 +1,26 @@
-<?php $pageTitle='Sequence: '.$data['name'];
+<?php
+/** @var array $data The object we are going to view */
+$pageTitle='Sequence: '.$data['name'];
 include realpath(__DIR__).'/../../core/_common/header.php'; ?>
 <script type="text/javascript">
 
-document.observe("dom:loaded",function(){
-	var grid=ui.grid();
+window.addEventListener('DOMContentLoaded',function(){
+    const grid = ui.grid();
 
-	var infoBox=grid.box({ 
+    grid.box({
 		classes:'r1 c1 h1 w1',
 		content:'This is only for testing and development. Go to the project page for a better way to work with proteins, constructs and sequences.'
 	});
 
-	var detailsBox=grid.box({ 
-		classes:'r2 c1 h2 w1',
-		title:'Details'
-	});
-
-	var frm=detailsBox.form({
-		action:'/api/construct/'+data['id'],
-		method:'patch',
-	});
-	frm.textField({ label:"Name", name:'name', value:data['name'] });
+    const detailsBox = grid.box({
+        classes: 'r2 c1 h2 w1',
+        title: 'Details'
+    });
+    const frm = detailsBox.form({
+        action: '/api/construct/' + data['id'],
+        method: 'patch',
+    });
+    frm.textField({ label:"Name", name:'name', value:data['name'] });
 	frm.textField({ label:"DNA Sequence", name:'dnasequence', value:'' });
 	frm.roleField({ 
 		label:"Construct",
@@ -33,4 +34,4 @@ document.observe("dom:loaded",function(){
 	
 });
 </script>
-<?php include realpath(__DIR__).'/../../core/_common/footer.php'; ?>
+<?php include realpath(__DIR__).'/../../core/_common/footer.php';
