@@ -1,5 +1,5 @@
 <p>To begin, tell IceBear where to keep uploaded files, images imported from imagers, etc.</p>
-<p>The pre-install script created and configured <strong>/icebearstore</strong> for you, but you can specify another location. This directory must exist and be writable by the Apache user (typically www-root).</p>
+<p>The pre-installation script created and configured <strong>/icebearstore</strong> for you, but you can specify another location. This directory must exist and be writable by the Apache user (typically www-root).</p>
 <label>Path to storage: <input name="livestore" type="text" value="/icebearstore"/></label>
 <p>IceBear will back up its database nightly if you specify a backup directory. Imported images and uploaded files will also be backed up into this directory as they are added to IceBear.</p>
 <p>If you specify a backup directory, it must exist and be writable by the Apache user (typically www-root); you must also specify how many nightly backups you wish to keep.</p>
