@@ -10,12 +10,12 @@ spl_autoload_register(function($className){
         'classes/core/authentication/',
         'classes/core/interface/',
         'classes/model/',
+        'classes/device/',
     );
     foreach($paths as $path){
         $filePath=__DIR__.'/../'.$path.$className.'.class.php';
         if(file_exists($filePath)){
-            /** @noinspection PhpIncludeInspection */
-            include_once($filePath);
+			include_once($filePath);
         }
     }
 });
